@@ -44,6 +44,10 @@ static void modif_buttons_update (GnomeDbDataWidgetInfo *info);
 static void data_widget_proxy_changed_cb (GnomeDbDataWidget *data_widget, GdaDataProxy *proxy, GnomeDbDataWidgetInfo *info);
 static void proxy_changed_cb (GdaDataProxy *proxy, GnomeDbDataWidgetInfo *info);
 static void proxy_sample_changed_cb (GdaDataProxy *proxy, gint sample_start, gint sample_end, GnomeDbDataWidgetInfo *info);
+static void proxy_row_changed_cb (GdaDataProxy *proxy, gint row, GnomeDbDataWidgetInfo *info);
+static void raw_grid_selection_changed_cb (GnomeDbRawGrid *grid, gboolean row_selected, 
+					   GnomeDbDataWidgetInfo *info);
+
 
 struct _GnomeDbDataWidgetInfoPriv
 {
@@ -108,7 +112,7 @@ gnome_db_data_widget_info_class_init (GnomeDbDataWidgetInfoClass * class)
         object_class->set_property = gnome_db_data_widget_info_set_property;
         object_class->get_property = gnome_db_data_widget_info_get_property;
 	g_object_class_install_property (object_class, PROP_DATA_WIDGET,
-                                         g_param_spec_pointer ("data_widget", NULL, NULL,
+                                         g_param_spec_object ("data_widget", NULL, NULL, GNOME_DB_TYPE_DATA_WIDGET,
                                                                G_PARAM_READABLE | G_PARAM_WRITABLE));
 	g_object_class_install_property (object_class, PROP_FLAGS,
                                          g_param_spec_uint ("flags", NULL, NULL, 0, G_MAXUINT, 
@@ -155,6 +159,10 @@ data_widget_destroyed_cb (GnomeDbDataWidget *wid, GnomeDbDataWidgetInfo *info)
 					      G_CALLBACK (data_widget_destroyed_cb), info);
 	g_signal_handlers_disconnect_by_func (G_OBJECT (wid),
 					      G_CALLBACK (data_widget_proxy_changed_cb), info);
+	if (IS_GNOME_DB_RAW_GRID (info->priv->data_widget))
+		g_signal_handlers_disconnect_by_func (info->priv->data_widget,
+						      G_CALLBACK (raw_grid_selection_changed_cb), info);
+
 	info->priv->data_widget = NULL;
 }
 
@@ -169,6 +177,8 @@ proxy_destroyed_cb (GdaDataProxy *proxy, GnomeDbDataWidgetInfo *info)
 					      G_CALLBACK (proxy_changed_cb), info);
 	g_signal_handlers_disconnect_by_func (G_OBJECT (proxy),
 					      G_CALLBACK (proxy_sample_changed_cb), info);
+	g_signal_handlers_disconnect_by_func (G_OBJECT (proxy),
+					      G_CALLBACK (proxy_row_changed_cb), info);
 	info->priv->proxy = NULL;
 }
 
@@ -235,7 +245,7 @@ gnome_db_data_widget_info_set_property (GObject *object,
 			if (info->priv->proxy)
 				proxy_destroyed_cb (info->priv->proxy, info);
 
-			info->priv->data_widget = g_value_get_pointer (value);
+			info->priv->data_widget = GNOME_DB_DATA_WIDGET(g_value_get_object (value));
 			if (info->priv->data_widget) {
 				GdaDataProxy *proxy;
 				GdaDataModelIter *iter;
@@ -245,6 +255,10 @@ gnome_db_data_widget_info_set_property (GObject *object,
 						  G_CALLBACK (data_widget_destroyed_cb), info);
 				g_signal_connect (info->priv->data_widget, "proxy_changed",
 						  G_CALLBACK (data_widget_proxy_changed_cb), info);
+				if (IS_GNOME_DB_RAW_GRID (info->priv->data_widget))
+					g_signal_connect (info->priv->data_widget, "selection_changed",
+							  G_CALLBACK (raw_grid_selection_changed_cb), info);
+
 				/* proxy */
 				proxy = gnome_db_data_widget_get_proxy (info->priv->data_widget);
 				if (proxy) {
@@ -255,6 +269,10 @@ gnome_db_data_widget_info_set_property (GObject *object,
 							  G_CALLBACK (proxy_changed_cb), info);
 					g_signal_connect (G_OBJECT (proxy), "sample_changed",
 							  G_CALLBACK (proxy_sample_changed_cb), info);
+					g_signal_connect (G_OBJECT (proxy), "row_inserted",
+							  G_CALLBACK (proxy_row_changed_cb), info);
+					g_signal_connect (G_OBJECT (proxy), "row_removed",
+							  G_CALLBACK (proxy_row_changed_cb), info);
 					
 					/* iter */
 					iter = gnome_db_data_widget_get_current_data (GNOME_DB_DATA_WIDGET 
@@ -267,6 +285,7 @@ gnome_db_data_widget_info_set_property (GObject *object,
 								  G_CALLBACK (iter_row_changed_cb), info);
 					}
 				}
+				modif_buttons_update (info);
 			}
                         break;
                 case PROP_FLAGS:
@@ -331,12 +350,24 @@ proxy_sample_changed_cb (GdaDataProxy *proxy, gint sample_start, gint sample_end
 	modif_buttons_update (info);
 }
 
+static void
+proxy_row_changed_cb (GdaDataProxy *proxy, gint row, GnomeDbDataWidgetInfo *info)
+{
+	modif_buttons_update (info);
+}
+
 static void
 iter_row_changed_cb (GdaDataModelIter *iter, gint row, GnomeDbDataWidgetInfo *info)
 {
 	modif_buttons_update (info);
 }
 
+static void
+raw_grid_selection_changed_cb (GnomeDbRawGrid *grid, gboolean row_selected, GnomeDbDataWidgetInfo *info)
+{
+	modif_buttons_update (info);
+}
+
 /*
  *
  * Modification buttons (Commit changes, Reset info, New entry, Delete)
@@ -419,14 +450,14 @@ modif_buttons_make (GnomeDbDataWidgetInfo *info)
 					  G_CALLBACK (row_spin_changed_cb), info);
 
 			/* rows counter */
-			wid = gtk_label_new (" / ?");
+			wid = gtk_label_new (" /?");
 			gtk_widget_show (wid);
 			info->priv->current_sample = wid;
 			gtk_box_pack_start (GTK_BOX (info), wid, FALSE, FALSE, 2);
 		}
 		else {
 			/* read-only counter (mainly for grids) */
-			wid = gtk_label_new ("? - ? / ?");
+			wid = gtk_label_new ("? - ? /?");
 			gtk_widget_show (wid);
 			info->priv->current_sample = wid;
 			gtk_box_pack_start (GTK_BOX (info), wid, FALSE, FALSE, 2);
@@ -455,48 +486,52 @@ static void
 modif_buttons_update (GnomeDbDataWidgetInfo *info)
 {
 	GdaDataModelIter *model_iter;
-	gboolean bool;
+	gboolean wrows;
 	gint has_selection;	
 	gint row;
+
 	gint proxy_rows = 0;
+	gint proxied_rows = 0;
+	gint all_rows = 0;
 
 	GtkAction *action;
-	gboolean params_valid;
 	gint sample_first_row = 0, sample_last_row = 0, sample_size = 0;
-	gint row_count = 0;
 	guint flags = 0;
 	
 	model_iter = gnome_db_data_widget_get_current_data (info->priv->data_widget);
-	params_valid = model_iter ? gda_parameter_list_is_valid (GDA_PARAMETER_LIST (model_iter)) : TRUE;
 	if (info->priv->proxy) {
 		proxy_rows = gda_data_model_get_n_rows (GDA_DATA_MODEL (info->priv->proxy));
+		proxied_rows = gda_data_proxy_get_proxied_model_n_rows (info->priv->proxy);
+		all_rows = proxied_rows + gda_data_proxy_get_n_new_rows (info->priv->proxy);
+
+		/* samples don't take into account the proxy's inserted rows */
 		sample_first_row = gda_data_proxy_get_sample_start (info->priv->proxy);
 		sample_last_row = gda_data_proxy_get_sample_end (info->priv->proxy);
-		row_count = gda_data_proxy_get_proxied_model_n_rows (info->priv->proxy);
-		flags = gda_data_model_get_access_flags (GDA_DATA_MODEL (info->priv->proxy));
 		sample_size = gda_data_proxy_get_sample_size (info->priv->proxy);
+
+		flags = gda_data_model_get_access_flags (GDA_DATA_MODEL (info->priv->proxy));
 	}
 
 	/* sensitiveness of the text indications and of the spin button */
-	bool = (proxy_rows <= 0) ? FALSE : TRUE;
+	wrows = (proxy_rows <= 0) ? FALSE : TRUE;
 	row = model_iter ? gda_data_model_iter_get_row (model_iter) : 0;
 	if (info->priv->flags & GNOME_DB_DATA_WIDGET_INFO_CURRENT_ROW) {
-		if (!bool) {
+		if (!wrows) {
 			if (info->priv->flags & GNOME_DB_DATA_WIDGET_INFO_ROW_MOVE_BUTTONS) {
 				BLOCK_SPIN;
 				gtk_spin_button_set_range (GTK_SPIN_BUTTON (info->priv->row_spin), 0, 1);
 				gtk_spin_button_set_value (GTK_SPIN_BUTTON (info->priv->row_spin), 0);
 				UNBLOCK_SPIN;
-				gtk_label_set_text (GTK_LABEL (info->priv->current_sample), " / ?");
+				gtk_label_set_text (GTK_LABEL (info->priv->current_sample), " /?");
 			}
 			else
-				gtk_label_set_text (GTK_LABEL (info->priv->current_sample), "? - ? / ?");
+				gtk_label_set_text (GTK_LABEL (info->priv->current_sample), "? - ? /?");
 		}
 		else {
 			gchar *str;
 			gint total;
 			
-			total = sample_last_row + 1 + gda_data_proxy_get_n_new_rows (info->priv->proxy);
+			total = sample_first_row + proxy_rows;
 			if (info->priv->flags & GNOME_DB_DATA_WIDGET_INFO_ROW_MOVE_BUTTONS) {
 				if (total <= 0)
 					str = g_strdup (" / 0");
@@ -504,30 +539,36 @@ modif_buttons_update (GnomeDbDataWidgetInfo *info)
 					str = g_strdup_printf (" / %d", proxy_rows);
 				BLOCK_SPIN;
 				gtk_spin_button_set_range (GTK_SPIN_BUTTON (info->priv->row_spin), 1, proxy_rows);
-				if (gtk_spin_button_get_value (GTK_SPIN_BUTTON (info->priv->row_spin)) != row+1)
-					gtk_spin_button_set_value (GTK_SPIN_BUTTON (info->priv->row_spin), row+1);
+				if (row >= 0)
+					if (gtk_spin_button_get_value (GTK_SPIN_BUTTON (info->priv->row_spin)) != row+1)
+						gtk_spin_button_set_value (GTK_SPIN_BUTTON (info->priv->row_spin), row+1);
 				UNBLOCK_SPIN;
 			}
 			else {
 				if (total <= 0)
 					str = g_strdup_printf ("0 - 0 / 0");
-				else
-					str = g_strdup_printf ("%d - %d / %d", 
-							       sample_first_row + 1, total, row_count);			
+				else {
+					if (all_rows < 0)
+						str = g_strdup_printf ("%d - %d /?", sample_first_row + 1, total);
+					else
+						str = g_strdup_printf ("%d - %d / %d", 
+								       sample_first_row + 1, total, all_rows);
+				}
 			}
 			gtk_label_set_text (GTK_LABEL (info->priv->current_sample), str);
 			g_free (str);
 		}
 
-		gtk_widget_set_sensitive (info->priv->current_sample, bool);
+		gtk_widget_set_sensitive (info->priv->current_sample, wrows);
 		if (info->priv->row_spin)
-			gtk_widget_set_sensitive (info->priv->row_spin, bool);
+			gtk_widget_set_sensitive (info->priv->row_spin, wrows && (row >= 0));
 	}
 
 	/* current row modifications */
 	if (info->priv->buttons_bar) {
 		gboolean changed = FALSE;
 		gboolean to_be_deleted = FALSE;
+		gboolean is_inserted = FALSE;
 		gboolean force_del_btn = FALSE;
 		gboolean force_undel_btn = FALSE;
 
@@ -535,43 +576,62 @@ modif_buttons_update (GnomeDbDataWidgetInfo *info)
 			changed = gda_data_proxy_has_changed (info->priv->proxy);
 			
 			has_selection = (row >= 0) ? TRUE : FALSE;
-			if (has_selection)
+			if (has_selection) {
 				to_be_deleted = gda_data_proxy_row_is_deleted (info->priv->proxy, row);
+				is_inserted = gda_data_proxy_row_is_inserted (info->priv->proxy, row);
+			}
 			else 
 				if (IS_GNOME_DB_RAW_GRID (info->priv->data_widget)) {
 					/* bad for encapsulation, but very usefull... */
 					GList *sel, *list;
 					
 					sel = gnome_db_raw_grid_get_selection ((GnomeDbRawGrid*) info->priv->data_widget);
-					list = sel;
-					while (list && (!force_del_btn || !force_undel_btn)) {
-						if (gda_data_proxy_row_is_deleted (info->priv->proxy, 
-										   GPOINTER_TO_INT (list->data)))
-							force_undel_btn = TRUE;
-						else
-							force_del_btn = TRUE;
-						list = g_list_next (list);
+					if (sel) {
+						list = sel;
+						while (list && (!force_del_btn || !force_undel_btn)) {
+							if ((GPOINTER_TO_INT (list->data) != -1) && 
+							    gda_data_proxy_row_is_deleted (info->priv->proxy, 
+											   GPOINTER_TO_INT (list->data)))
+								force_undel_btn = TRUE;
+							else
+								force_del_btn = TRUE;
+							list = g_list_next (list);
+						}
+						list = sel;
+
+						is_inserted = TRUE;
+						while (list && is_inserted) {
+							if (GPOINTER_TO_INT (list->data) != -1)
+								is_inserted = FALSE;
+							list = g_list_next (list);
+						}
+						g_list_free (sel);
 					}
-					g_list_free (list);
 				}
 		}
 
-		action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionCommit");
-		g_object_set (G_OBJECT (action), "sensitive", changed ? TRUE : FALSE, NULL);
-		
-		action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionReset");
-		g_object_set (G_OBJECT (action), "sensitive", changed ? TRUE : FALSE, NULL);
-		
-		action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionNew");
-		g_object_set (G_OBJECT (action), "sensitive", flags & GDA_DATA_MODEL_ACCESS_INSERT ? TRUE : FALSE, NULL);
-		
-		action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionDelete");
-		bool = (flags & GDA_DATA_MODEL_ACCESS_DELETE) && (force_del_btn || (! to_be_deleted && has_selection));
-		g_object_set (G_OBJECT (action), "sensitive", bool, NULL);
-
-		action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionUndelete");
-		bool = (flags & GDA_DATA_MODEL_ACCESS_DELETE) && (force_undel_btn || (to_be_deleted && has_selection));
-		g_object_set (G_OBJECT (action), "sensitive", bool, NULL);
+		if (info->priv->flags & GNOME_DB_DATA_WIDGET_INFO_ROW_MODIF_BUTTONS) {
+			action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionCommit");
+			g_object_set (G_OBJECT (action), "sensitive", changed ? TRUE : FALSE, NULL);
+			
+			action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionReset");
+			g_object_set (G_OBJECT (action), "sensitive", changed ? TRUE : FALSE, NULL);
+			
+			action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionNew");
+			g_object_set (G_OBJECT (action), "sensitive", 
+				      flags & GDA_DATA_MODEL_ACCESS_INSERT ? TRUE : FALSE, NULL);
+			
+			action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionDelete");
+			wrows = is_inserted ||
+				((flags & GDA_DATA_MODEL_ACCESS_DELETE) && 
+				 (force_del_btn || (! to_be_deleted && has_selection)));
+			g_object_set (G_OBJECT (action), "sensitive", wrows, NULL);
+			
+			action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionUndelete");
+			wrows = (flags & GDA_DATA_MODEL_ACCESS_DELETE) && 
+				(force_undel_btn || (to_be_deleted && has_selection));
+			g_object_set (G_OBJECT (action), "sensitive", wrows, NULL);
+		}
 	}
 
 	/* current row moving */
@@ -581,26 +641,42 @@ modif_buttons_update (GnomeDbDataWidgetInfo *info)
 		action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionPrevRecord");
 		g_object_set (G_OBJECT (action), "sensitive", (row <= 0) ? FALSE : TRUE, NULL);
 		action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionNextRecord");
-		g_object_set (G_OBJECT (action), "sensitive", (row == proxy_rows -1) ? FALSE : TRUE, NULL);
+		g_object_set (G_OBJECT (action), "sensitive", (row == proxy_rows -1) || (row < 0) ? FALSE : TRUE, 
+			      NULL);
 		action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionLastRecord");
-		g_object_set (G_OBJECT (action), "sensitive", (row == proxy_rows -1) ? FALSE : TRUE, NULL);
+		g_object_set (G_OBJECT (action), "sensitive", (row == proxy_rows -1) || (row < 0) ? FALSE : TRUE, 
+			      NULL);
 	}
 
 	/* chunck indications */
 	if (info->priv->flags & GNOME_DB_DATA_WIDGET_INFO_CHUNCK_CHANGE_BUTTONS) {
-		bool = (sample_size > 0) ? TRUE : FALSE;
+		gboolean abool;
+		wrows = (sample_size > 0) ? TRUE : FALSE;
 		action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionFirstChunck");
-		g_object_set (G_OBJECT (action), "sensitive", bool && sample_first_row > 0 ? TRUE : FALSE, NULL);
+		g_object_set (G_OBJECT (action), "sensitive", wrows && sample_first_row > 0 ? TRUE : FALSE, NULL);
 		action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionPrevChunck");
-		g_object_set (G_OBJECT (action), "sensitive", bool && sample_first_row > 0 ? TRUE : FALSE, NULL);
+		g_object_set (G_OBJECT (action), "sensitive", wrows && sample_first_row > 0 ? TRUE : FALSE, NULL);
 		action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionNextChunck");
-		g_object_set (G_OBJECT (action), "sensitive", bool && sample_last_row < row_count - 1 ? TRUE : FALSE, 
-			      NULL);
+		abool = (proxied_rows != -1) ? wrows && (sample_last_row < proxied_rows - 1) : TRUE;
+		g_object_set (G_OBJECT (action), "sensitive", abool, NULL);
 		action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionLastChunck");
-		g_object_set (G_OBJECT (action), "sensitive", bool && sample_last_row < row_count - 1 ? TRUE : FALSE, 
-			      NULL);
+		g_object_set (G_OBJECT (action), "sensitive", wrows && (sample_last_row < proxied_rows - 1), NULL);
 	}
 	
+	if (info->priv->proxy && (info->priv->flags & GNOME_DB_DATA_WIDGET_INFO_ROW_MODIF_BUTTONS)) {
+		gboolean autocommit;
+
+		g_object_get (G_OBJECT (info->priv->proxy), "autocommit", &autocommit, NULL);
+		action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionUndelete");
+		gtk_action_set_visible (action, !autocommit);
+
+		action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionCommit");
+		gtk_action_set_visible (action, !autocommit);
+
+		action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionReset");
+		gtk_action_set_visible (action, !autocommit);
+	}
+
 	if (info->priv->uimanager)
 		gtk_ui_manager_ensure_update (info->priv->uimanager); 
 }
