@@ -1,6 +1,6 @@
 /* gnome-db-decl.h
  *
- * Copyright (C) 2003 - 2005 Vivien Malerba
+ * Copyright (C) 2003 - 2006 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -35,13 +35,6 @@ typedef struct _GnomeDbSelector      GnomeDbSelector;
 typedef struct _GnomeDbSelectorClass GnomeDbSelectorClass;
 typedef struct _GnomeDbSelectorPriv  GnomeDbSelectorPriv;
 
-/*
- * Server part
- */
-typedef struct _GnomeDbResultSet GnomeDbResultSet;
-typedef struct _GnomeDbResultSetClass GnomeDbResultSetClass;
-typedef struct _GnomeDbResultSetPrivate GnomeDbResultSetPrivate;
-
 /* 
  * Interfaces 
  */
@@ -87,6 +80,9 @@ typedef struct _GnomeDbCanvasFieldPrivate GnomeDbCanvasFieldPrivate;
 #define GNOME_DB_COLOR_NORMAL_INVALID "#ff6a6a"
 #define GNOME_DB_COLOR_PRELIGHT_INVALID "#ff7b7b"
 
+#define GNOME_DB_REFERER_ACTIVE   "#e0ffe0"
+#define GNOME_DB_REFERER_INACTIVE "#ffe0e0"
+#define GNOME_DB_REFERER_UNKNOWN  "#e0e0ff"
 
 /*
  * Variou macros
