@@ -33,7 +33,7 @@
 #include <libgnomedb/gnome-db-dsn-spec.h>
 #include <libgnomedb/gnome-db-provider-selector.h>
 #include <libgnomedb/gnome-db-util.h>
-#include <libgnomedb/gnome-db-basic-form.h>
+#include <libgnomedb/gnome-db-server-operation.h>
 #include <glib/gi18n-lib.h>
 
 #define PARENT_TYPE GNOME_TYPE_DRUID
@@ -41,7 +41,7 @@
 struct _GnomeDbDsnConfigDruidPrivate {
 	GdaClient         *client;
 	GdaDataSourceInfo *dsn_info;
-	GdaParameterList  *paramlist;
+	GdaServerOperation *create_db_op;
 
 	GdkPixbuf *logo;
 
@@ -55,7 +55,6 @@ struct _GnomeDbDsnConfigDruidPrivate {
 	GtkWidget *general_username;
 	GtkWidget *general_password;
 	GtkWidget *general_is_global;
-	gchar     *create_db_spec;
 
 	GtkWidget *choose_page;
 	GtkWidget *choose_toggle;
@@ -88,20 +87,20 @@ static GObjectClass *parent_class = NULL;
  * Callbacks
  */
 
-static gchar *
+static GdaServerOperation *
 get_specs_database_creation (GnomeDbDsnConfigDruid *druid)
 {
-	if (! druid->priv->create_db_spec) {
+	if (! druid->priv->create_db_op) {
 		if (!druid->priv->client)
 			druid->priv->client = gda_client_new ();
 		
-		druid->priv->create_db_spec = gda_client_get_provider_specs (druid->priv->client, 
-								    	     gnome_db_provider_selector_get_selected_provider (
-								             GNOME_DB_PROVIDER_SELECTOR (druid->priv->general_provider)),
-								             GDA_CLIENT_SPECS_CREATE_DATABASE);
+		druid->priv->create_db_op = 
+			gda_client_prepare_create_database (druid->priv->client, 
+					  gnome_db_provider_selector_get_selected_provider (
+					  GNOME_DB_PROVIDER_SELECTOR (druid->priv->general_provider)));
 	}
 
-	return druid->priv->create_db_spec;
+	return druid->priv->create_db_op;
 }
 
 static void
@@ -129,27 +128,20 @@ druid_finished_cb (GnomeDruidPage *druid_page, GtkWidget *druid_widget, GnomeDbD
 
 	/* New database creation first */
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (druid->priv->choose_toggle))) {
-		if (!gda_parameter_list_is_valid (druid->priv->paramlist)) {
+		if (!gda_server_operation_is_valid (druid->priv->create_db_op, NULL, NULL)) {
 			gnome_db_show_error (NULL, _("Missing mandatory informmation, to create database"));
 			gnome_druid_set_page (GNOME_DRUID (druid), GNOME_DRUID_PAGE (druid->priv->newdb_page));
 			return;
 		}
 		else {
-			GSList *params;	
 			GdaProviderInfo *prov_info;
 			GSList *dsn_params;
 			GError *error = NULL;
 
-			params = druid->priv->paramlist->parameters;
-			while (params) {
-				gda_object_set_name (GDA_OBJECT (params->data), gda_object_get_id (GDA_OBJECT (params->data)));
-				params = g_slist_next (params);
-			}
-			
-			allok = gda_client_create_database (druid->priv->client, 
-							    gnome_db_provider_selector_get_selected_provider (
-								    GNOME_DB_PROVIDER_SELECTOR (druid->priv->general_provider)),
-							    druid->priv->paramlist, &error);
+			allok = gda_client_perform_create_database (druid->priv->client, 
+					       gnome_db_provider_selector_get_selected_provider (
+					       GNOME_DB_PROVIDER_SELECTOR (druid->priv->general_provider)),
+								    druid->priv->create_db_op, &error);
 			if (!allok) {
 				gchar *str;
 				str = g_strdup_printf (_("Error creating database: %s"), 
@@ -162,25 +154,33 @@ druid_finished_cb (GnomeDruidPage *druid_page, GtkWidget *druid_widget, GnomeDbD
 			}
 			
 			/* make the connection string for the data source */
-			prov_info = gda_config_get_provider_by_name (gnome_db_provider_selector_get_selected_provider (
-									     GNOME_DB_PROVIDER_SELECTOR (druid->priv->general_provider)));
+			prov_info = gda_config_get_provider_by_name (
+                                             gnome_db_provider_selector_get_selected_provider (
+					     GNOME_DB_PROVIDER_SELECTOR (druid->priv->general_provider)));
 			g_return_if_fail (prov_info);
 			dsn_params = prov_info->gda_params->parameters;
 			while (dsn_params) {
-				GdaParameter *param = GDA_PARAMETER (dsn_params->data);				
-				const GdaValue *value;
+				GdaParameter *param = GDA_PARAMETER (dsn_params->data);
+				const GValue *value;
 				
-				value = gda_parameter_get_value (param);
-				if (value && !gda_value_is_null ((GdaValue *) value)) {
+				value = gda_server_operation_get_value_at (druid->priv->create_db_op,
+									   "/DB_DEF_P/%s",
+									   gda_object_get_id (GDA_OBJECT (param)));
+				if (!value)
+					value = gda_server_operation_get_value_at (druid->priv->create_db_op,
+									   "/SERVER_CNX_P/%s",
+									   gda_object_get_id (GDA_OBJECT (param)));
+				
+				if (value && !gda_value_is_null ((GValue *) value)) {
 					gchar *str;
 
 					if (dsn_params == prov_info->gda_params->parameters)
 						cnc_string = g_string_new ("");
 					else
-						g_string_append (cnc_string, "; ");
-					str = gda_value_stringify ((GdaValue *) value);
+						g_string_append (cnc_string, ";");
+					str = gda_value_stringify ((GValue *) value);
 					g_string_append_printf (cnc_string, "%s=%s",
-								gda_object_get_name (GDA_OBJECT (param)), str);
+								gda_object_get_id (GDA_OBJECT (param)), str);
 					g_free (str);
 				}
 				
@@ -193,8 +193,9 @@ druid_finished_cb (GnomeDruidPage *druid_page, GtkWidget *druid_widget, GnomeDbD
 	if (allok) {
 		druid->priv->dsn_info = g_new0 (GdaDataSourceInfo, 1);
 		druid->priv->dsn_info->name = g_strdup (gtk_entry_get_text (GTK_ENTRY (druid->priv->general_name)));
-		druid->priv->dsn_info->provider = g_strdup (gnome_db_provider_selector_get_selected_provider (
-								    GNOME_DB_PROVIDER_SELECTOR (druid->priv->general_provider)));
+		druid->priv->dsn_info->provider = g_strdup (
+					 gnome_db_provider_selector_get_selected_provider (
+					 GNOME_DB_PROVIDER_SELECTOR (druid->priv->general_provider)));
 		if (cnc_string) {
 			druid->priv->dsn_info->cnc_string = cnc_string->str;
 			g_string_free (cnc_string, FALSE);
@@ -231,9 +232,9 @@ provider_changed_cb (GtkWidget *combo, GnomeDbDsnConfigDruid *druid)
 	}
 
 	/* is the database creation supported by the chosen provider? */
-	if (druid->priv->create_db_spec) {
-		g_free (druid->priv->create_db_spec);
-		druid->priv->create_db_spec = NULL;
+	if (druid->priv->create_db_op) {
+		g_object_unref (druid->priv->create_db_op);
+		druid->priv->create_db_op = NULL;
 	}
 }
 
@@ -243,6 +244,7 @@ general_next_pressed_cb (GnomeDruidPage *druid_page,
 			 GnomeDbDsnConfigDruid *druid)
 {
 	GdaDataSourceInfo *dsn_info;
+	GdaServerOperation *op;
 	GdaProviderInfo *prov_info;
 	gint i;
 	const gchar *name;
@@ -316,8 +318,8 @@ general_next_pressed_cb (GnomeDruidPage *druid_page,
 		gnome_db_dsn_spec_set_provider (GNOME_DB_DSN_SPEC (druid->priv->provider_detail), provider);
 
 
-	str = get_specs_database_creation (druid);
-	if (str) 
+	op = get_specs_database_creation (druid);
+	if (op) 
 		return FALSE;
 	else {
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (druid->priv->choose_toggle), FALSE);
@@ -334,40 +336,20 @@ choose_next_pressed_cb (GnomeDruidPage *druid_page,
 	g_return_val_if_fail (GNOME_DB_IS_DSN_CONFIG_DRUID (druid), FALSE);
 
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (druid->priv->choose_toggle))) {
-		/* prepare a new data set for database creation parameters */
-		gchar *str;
-
 		if (! druid->priv->newdb_params) {
 			if (!druid->priv->client)
 				druid->priv->client = gda_client_new ();
+			GdaServerOperation *op;
 			
-			str = gda_client_get_provider_specs (druid->priv->client, 
-						             gnome_db_provider_selector_get_selected_provider (
-						             GNOME_DB_PROVIDER_SELECTOR (druid->priv->general_provider)),
-						             GDA_CLIENT_SPECS_CREATE_DATABASE);
-			if (str) {
-				GdaParameterList *dset;
+			op = get_specs_database_creation (druid);
+			if (op) {
 				GError *error = NULL;
-				
-				dset = GDA_PARAMETER_LIST (gda_parameter_list_new_from_spec (NULL, str, &error));
-				if (dset) {
-					druid->priv->paramlist = dset;
-					druid->priv->newdb_params = gnome_db_basic_form_new (dset);
-					gnome_db_basic_form_show_entries_actions (GNOME_DB_BASIC_FORM (druid->priv->newdb_params),
-										  FALSE);
-				}
-				else {
-					gchar *str2;
-					str2 = g_strdup_printf ("Provider internal error: %s", 
-								error && error->message ? error->message : "???" );
-					druid->priv->newdb_params = gtk_label_new (str2);
-					g_free (str2);
-				}
-				
+
+				druid->priv->newdb_params = gnome_db_server_operation_new (op);
 				gtk_widget_show (druid->priv->newdb_params);
-				gtk_box_pack_start (GTK_BOX (druid->priv->newdb_box), druid->priv->newdb_params,
+				gtk_box_pack_start (GTK_BOX (druid->priv->newdb_box), 
+						    druid->priv->newdb_params,
 						    TRUE, TRUE, 0);
-				g_free (str);
 			}
 		}
 		gnome_druid_set_page (GNOME_DRUID (druid), GNOME_DRUID_PAGE (druid->priv->newdb_page));
@@ -395,15 +377,15 @@ provider_back_pressed_cb (GnomeDruidPage *druid_page,
 			  GtkWidget *druid_widget,
 			  GnomeDbDsnConfigDruid *druid)
 {
-	gchar *str;
+	GdaServerOperation *op;
 	g_return_val_if_fail (GNOME_DB_IS_DSN_CONFIG_DRUID (druid), TRUE);
 
 
 	/*
 	 * is the database creation supported by the chosen provider?
 	 */
-	str = get_specs_database_creation (druid);
-	if (str) {
+	op = get_specs_database_creation (druid);
+	if (op) {
 		/* database creation is supported */
 		gnome_druid_set_page (GNOME_DRUID (druid), GNOME_DRUID_PAGE (druid->priv->choose_page));
 	}
@@ -464,7 +446,7 @@ gnome_db_dsn_config_druid_init (GnomeDbDsnConfigDruid *druid,
 	druid->priv = g_new0 (GnomeDbDsnConfigDruidPrivate, 1);
 	druid->priv->dsn_info = g_new0 (GdaDataSourceInfo, 1);
 	druid->priv->provider_detail = NULL;
-	druid->priv->create_db_spec = NULL;
+	druid->priv->create_db_op = NULL;
 
 	/* load icons */
 	druid->priv->logo = gdk_pixbuf_new_from_file (LIBGNOMEDB_ICONSDIR "/gnome-db.png", NULL);
@@ -675,11 +657,8 @@ gnome_db_dsn_config_druid_finalize (GObject *object)
 	if (druid->priv->dsn_info)
 		gda_data_source_info_free (druid->priv->dsn_info);
 
-	if (druid->priv->create_db_spec)
-		g_free (druid->priv->create_db_spec);
-
-	if (druid->priv->paramlist)
-		g_object_unref (druid->priv->paramlist);
+	if (druid->priv->create_db_op)
+		g_object_unref (druid->priv->create_db_op);
 
 	g_free (druid->priv);
 	druid->priv = NULL;
