@@ -367,7 +367,7 @@ params_to_string (GnomeDbDsnSpec *spec)
 				GdaDataHandler *dh;
 				GType dtype;
 
-				dtype = gda_parameter_get_gda_type (param);
+				dtype = gda_parameter_get_g_type (param);
 				dh = gda_dict_get_default_handler (default_dict, dtype);
 				str = gda_data_handler_get_str_from_value (dh, value);
 			}
