@@ -90,19 +90,40 @@ gnome_db_grid_class_init (GnomeDbGridClass *class)
         object_class->set_property = gnome_db_grid_set_property;
         object_class->get_property = gnome_db_grid_get_property;
 	g_object_class_install_property (object_class, PROP_RAW_GRID,
-                                         g_param_spec_pointer ("raw_grid", NULL, NULL,
-                                                               G_PARAM_READABLE));
+                                         g_param_spec_object ("raw_grid", NULL, NULL, 
+							      GNOME_DB_TYPE_RAW_GRID,
+							      G_PARAM_READABLE));
 	g_object_class_install_property (object_class, PROP_INFO,
-                                         g_param_spec_pointer ("widget_info", NULL, NULL,
-							       G_PARAM_READABLE));
+                                         g_param_spec_object ("widget_info", NULL, NULL, 
+							      GNOME_DB_TYPE_DATA_WIDGET_INFO,
+							      G_PARAM_READABLE));
 }
 
 static void
-gnome_db_grid_init (GnomeDbGrid *wid)
+gnome_db_grid_init (GnomeDbGrid *grid)
 {
-	wid->priv = g_new0 (GnomeDbGridPriv, 1);
-	wid->priv->raw_grid = NULL;
-	wid->priv->info = NULL;
+	GtkWidget *sw;
+	
+	grid->priv = g_new0 (GnomeDbGridPriv, 1);
+	grid->priv->raw_grid = NULL;
+	grid->priv->info = NULL;
+	
+	sw = gtk_scrolled_window_new (NULL, NULL);
+        gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+        gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw), GTK_SHADOW_IN);
+	gtk_box_pack_start (GTK_BOX (grid), sw, TRUE, TRUE, 0);
+	gtk_widget_show (sw);
+
+	grid->priv->raw_grid = gnome_db_raw_grid_new (NULL);
+	gtk_container_add (GTK_CONTAINER (sw), grid->priv->raw_grid);
+	gtk_widget_show (grid->priv->raw_grid);
+
+	grid->priv->info = gnome_db_data_widget_info_new (GNOME_DB_DATA_WIDGET (grid->priv->raw_grid), 
+							  GNOME_DB_DATA_WIDGET_INFO_CURRENT_ROW |
+							  GNOME_DB_DATA_WIDGET_INFO_ROW_MODIF_BUTTONS |
+							  GNOME_DB_DATA_WIDGET_INFO_CHUNCK_CHANGE_BUTTONS);
+	gtk_box_pack_start (GTK_BOX (grid), grid->priv->info, FALSE, TRUE, 0);
+	gtk_widget_show (grid->priv->info);
 }
 
 /**
@@ -117,28 +138,12 @@ GtkWidget *
 gnome_db_grid_new (GdaDataModel *model)
 {
 	GnomeDbGrid *grid;
-	GtkWidget *sw;
-
+	
 	g_return_val_if_fail (!model || GDA_IS_DATA_MODEL (model), NULL);
 
 	grid = (GnomeDbGrid *) g_object_new (GNOME_DB_TYPE_GRID, NULL);
 
-	sw = gtk_scrolled_window_new (NULL, NULL);
-        gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-        gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw), GTK_SHADOW_IN);
-	gtk_box_pack_start (GTK_BOX (grid), sw, TRUE, TRUE, 0);
-	gtk_widget_show (sw);
-
-	grid->priv->raw_grid = gnome_db_raw_grid_new (model);
-	gtk_container_add (GTK_CONTAINER (sw), grid->priv->raw_grid);
-	gtk_widget_show (grid->priv->raw_grid);
-
-	grid->priv->info = gnome_db_data_widget_info_new (GNOME_DB_DATA_WIDGET (grid->priv->raw_grid), 
-							  GNOME_DB_DATA_WIDGET_INFO_CURRENT_ROW |
-							  GNOME_DB_DATA_WIDGET_INFO_ROW_MODIF_BUTTONS |
-							  GNOME_DB_DATA_WIDGET_INFO_CHUNCK_CHANGE_BUTTONS);
-	gtk_box_pack_start (GTK_BOX (grid), grid->priv->info, FALSE, TRUE, 0);
-	gtk_widget_show (grid->priv->info);
+	g_object_set(grid->priv->raw_grid, "model", model, NULL);
 
 	return (GtkWidget *) grid;
 }
@@ -151,15 +156,15 @@ gnome_db_grid_set_property (GObject *object,
 				 GParamSpec *pspec)
 {
 	GnomeDbGrid *grid;
-
-        grid = GNOME_DB_GRID (object);
-        if (grid->priv) {
-                switch (param_id) {
-		default:
-			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
-			break;
-                }
-        }
+	
+	grid = GNOME_DB_GRID (object);
+	
+	switch (param_id) {
+		
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
+		break;
+	}
 }
 
 static void
@@ -170,20 +175,19 @@ gnome_db_grid_get_property (GObject *object,
 {
 	GnomeDbGrid *grid;
 
-        grid = GNOME_DB_GRID (object);
-        if (grid->priv) {
-                switch (param_id) {
-                case PROP_RAW_GRID:
-			g_value_set_pointer (value, grid->priv->raw_grid);
-                        break;
-                case PROP_INFO:
-			g_value_set_pointer (value, grid->priv->info);
-                        break;
-		default:
-			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
-			break;
-                }
-        }	
+	grid = GNOME_DB_GRID (object);
+	
+	switch (param_id) {
+	case PROP_RAW_GRID:
+		g_value_set_object (value, grid->priv->raw_grid);
+		break;
+	case PROP_INFO:
+		g_value_set_object (value, grid->priv->info);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
+		break;
+	}	
 }
 
 /**
