@@ -1,5 +1,5 @@
 /* GNOME DB library
- * Copyright (C) 1999 - 2004 The GNOME Foundation.
+ * Copyright (C) 1999 - 2006 The GNOME Foundation.
  *
  * AUTHORS:
  *      Rodrigo Moya <rodrigo@gnome-db.org>
@@ -21,11 +21,11 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include "gnome-db-decl.h"
 #include <libgda/gda-config.h>
-#include <libgnomedb/gnome-db-config.h>
 #include <libgnomedb/gnome-db-data-source-selector.h>
 #include <libgnomedb/gnome-db-dsn-config-druid.h>
-#include <libgnomedb/gnome-db-i18n.h>
+#include <glib/gi18n-lib.h>
 #include <libgnomedb/gnome-db-login.h>
 #include <libgnomedb/gnome-db-util.h>
 #include <gtk/gtklabel.h>
@@ -59,11 +59,23 @@ enum {
 	PROP_0,
 	PROP_DSN,
 	PROP_USERNAME,
-	PROP_PASSWORD
+	PROP_PASSWORD,
+	PROP_HIDING_BUTTON,
+	PROP_HIDING_DSN
 };
 
 static GObjectClass *parent_class = NULL;
 
+
+static void
+druid_finished_cb (GnomeDbDsnConfigDruid *druid, gboolean error, GnomeDbLogin *login);
+static void
+add_dsn_cb (GtkButton *button, GnomeDbLogin *login);
+static void
+dsn_entry_changed_cb (GnomeDbDataSourceSelector *sel, GnomeDbLogin *login);
+
+
+
 /*
  * Private functions
  */
@@ -93,16 +105,79 @@ gnome_db_login_class_init (GnomeDbLoginClass *klass)
 	g_object_class_install_property (
 		object_class, PROP_PASSWORD,
 		g_param_spec_string ("password", NULL, NULL, NULL, G_PARAM_READABLE));
+	g_object_class_install_property (
+		object_class, PROP_HIDING_BUTTON,
+		g_param_spec_boolean("hiding_button", NULL, NULL, TRUE, G_PARAM_READWRITE));
+	g_object_class_install_property (
+		object_class, PROP_HIDING_DSN,
+		g_param_spec_boolean("hiding_dsn", NULL, NULL, FALSE, G_PARAM_READWRITE));
+		
 }
 
 static void
 gnome_db_login_init (GnomeDbLogin *login, GnomeDbLoginClass *klass)
 {
+	GtkWidget *table;
+	GtkWidget *label;
+	
 	/* allocate the internal structure */
 	login->priv = g_new0 (GnomeDbLoginPrivate, 1);
+	
+	/* Init the properties*/
 	login->priv->dsn_name = NULL;
 	login->priv->hiding_button = TRUE;
 	login->priv->hiding_dsn = FALSE;
+	
+	/*construct the widget*/
+	
+	table = gnome_db_new_table_widget (3, 3, FALSE);
+	gtk_box_pack_start (GTK_BOX (login), table, TRUE, TRUE, 0);
+	
+	/* Create the Label for the data source selector*/
+	label = gnome_db_new_label_widget (_("Data Source:"));
+        gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
+	gtk_label_set_selectable (GTK_LABEL (label), FALSE);
+        gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
+	login->priv->dsn_label = label;
+	
+	/* Create the DSN selector*/
+	login->priv->dsn_entry = gnome_db_data_source_selector_new (login->priv->dsn_name);
+	gtk_widget_show (login->priv->dsn_entry); /* Show the DSN selector */
+	gtk_table_attach (GTK_TABLE (table), login->priv->dsn_entry, 1, 2, 0, 1,
+			  GTK_FILL | GTK_EXPAND | GTK_SHRINK, GTK_FILL, 0, 0);
+	g_signal_connect (G_OBJECT (login->priv->dsn_entry), "changed",
+			  G_CALLBACK (dsn_entry_changed_cb), login);
+			  
+	/* Create the DSN add button */
+	login->priv->create_dsn_button = gnome_db_new_button_widget_from_stock (GTK_STOCK_ADD);
+	g_signal_connect (G_OBJECT (login->priv->create_dsn_button), "clicked",
+			  G_CALLBACK (add_dsn_cb), login);
+	gtk_widget_hide (login->priv->create_dsn_button); /* Hide the create DSN button */
+	gtk_table_attach (GTK_TABLE (table), login->priv->create_dsn_button, 2, 3, 0, 1,
+			  GTK_FILL, GTK_FILL, 0, 0);
+			  
+	/* Create the Username Entry*/
+	label = gnome_db_new_label_widget (_("Username:"));
+        gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
+	gtk_label_set_selectable (GTK_LABEL (label), FALSE);
+        gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
+	login->priv->username_entry = gnome_db_new_entry_widget (0, TRUE);
+	gtk_table_attach (GTK_TABLE (table), login->priv->username_entry, 1, 3, 1, 2,
+			  GTK_FILL | GTK_EXPAND | GTK_SHRINK, GTK_FILL, 0, 0);			
+
+	/* Create the Password Entry*/
+	label = gnome_db_new_label_widget (_("Password:"));
+        gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
+	gtk_label_set_selectable (GTK_LABEL (label), FALSE);
+        gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
+	login->priv->password_entry = gnome_db_new_entry_widget (0, TRUE);
+	gtk_entry_set_visibility (GTK_ENTRY (login->priv->password_entry), FALSE);
+	gtk_table_attach (GTK_TABLE (table), login->priv->password_entry, 1, 3, 2, 3,
+			  GTK_FILL | GTK_EXPAND | GTK_SHRINK, GTK_FILL, 0, 0);
+
 }
 
 static void
@@ -116,6 +191,13 @@ gnome_db_login_set_property (GObject *object,
 	g_return_if_fail (GNOME_DB_IS_LOGIN (login));
 
 	switch (param_id) {
+	
+	case PROP_HIDING_BUTTON:
+		gnome_db_login_set_enable_create_button(login, g_value_get_boolean(value));
+		break;
+	case PROP_HIDING_DSN:
+		gnome_db_login_set_show_dsn_selector(login, g_value_get_boolean(value));
+		break;
 	default :
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -142,6 +224,12 @@ gnome_db_login_get_property (GObject *object,
 	case PROP_PASSWORD :
 		g_value_set_string (value, gnome_db_login_get_password (login));
 		break;
+	case PROP_HIDING_BUTTON:
+		g_value_set_boolean(value, gnome_db_login_get_enable_create_button(login));
+		break;
+	case PROP_HIDING_DSN:
+		g_value_set_boolean(value, gnome_db_login_get_show_dsn_selector(login));
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -192,20 +280,6 @@ gnome_db_login_get_type (void)
 static void
 druid_finished_cb (GnomeDbDsnConfigDruid *druid, gboolean error, GnomeDbLogin *login)
 {
-	const GdaDataSourceInfo *dsn_info;
-
-	if (!error) {
-		dsn_info = gnome_db_dsn_config_druid_get_dsn (druid);
-		if (dsn_info) {
-			gnome_db_config_save_data_source (dsn_info->name,
-							  dsn_info->provider,
-							  dsn_info->cnc_string,
-							  dsn_info->description,
-							  dsn_info->username,
-							  dsn_info->password);
-		}
-	}
-
 	gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (druid)));
 	gtk_widget_set_sensitive (gtk_widget_get_toplevel (GTK_WIDGET (login)), TRUE);
 }
@@ -238,62 +312,29 @@ add_dsn_cb (GtkButton *button, GnomeDbLogin *login)
 	gtk_widget_show (dialog);
 }
 
-static GtkWidget *
-gnome_db_login_construct (GnomeDbLogin *login, const gchar *dsn)
+static void
+dsn_entry_changed_cb (GnomeDbDataSourceSelector *sel, GnomeDbLogin *login)
 {
-	GtkWidget *table;
-	GtkWidget *label;
+	gchar *dsn;
+	GdaDataSourceInfo *info = NULL;
+        
+        dsn = gnome_db_data_source_selector_get_dsn(sel);
+        
+	info = gda_config_find_data_source (dsn);
 
-	g_return_val_if_fail (GNOME_DB_IS_LOGIN (login), GTK_WIDGET (login));
+	gtk_entry_set_text (GTK_ENTRY (login->priv->username_entry), "");
+	gtk_entry_set_text (GTK_ENTRY (login->priv->password_entry), "");
 
-	if (dsn != NULL)
-		login->priv->dsn_name = g_strdup (dsn);
-	else
-		login->priv->dsn_name = NULL;
-
-	table = gnome_db_new_table_widget (3, 3, FALSE);
-	gtk_box_pack_start (GTK_BOX (login), table, TRUE, TRUE, 0);
-
-	label = gnome_db_new_label_widget (_("Data Source:"));
-        gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
-	gtk_label_set_selectable (GTK_LABEL (label), FALSE);
-        gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
-	login->priv->dsn_label = label;
-	login->priv->dsn_entry = gnome_db_data_source_selector_new (dsn);
-	gtk_widget_show (login->priv->dsn_entry);
-	gtk_table_attach (GTK_TABLE (table), login->priv->dsn_entry, 1, 2, 0, 1,
-			  GTK_FILL | GTK_EXPAND | GTK_SHRINK, GTK_FILL, 0, 0);
-
-	login->priv->create_dsn_button = gnome_db_new_button_widget_from_stock (GTK_STOCK_ADD);
-	g_signal_connect (G_OBJECT (login->priv->create_dsn_button), "clicked",
-			  G_CALLBACK (add_dsn_cb), login);
-	gtk_widget_hide (login->priv->create_dsn_button);
-	gtk_table_attach (GTK_TABLE (table), login->priv->create_dsn_button, 2, 3, 0, 1,
-			  GTK_FILL, GTK_FILL, 0, 0);
-
-	label = gnome_db_new_label_widget (_("Username:"));
-        gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
-	gtk_label_set_selectable (GTK_LABEL (label), FALSE);
-        gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
-	login->priv->username_entry = gnome_db_new_entry_widget (0, TRUE);
-	gtk_table_attach (GTK_TABLE (table), login->priv->username_entry, 1, 3, 1, 2,
-			  GTK_FILL | GTK_EXPAND | GTK_SHRINK, GTK_FILL, 0, 0);
-
-	label = gnome_db_new_label_widget (_("Password:"));
-        gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
-	gtk_label_set_selectable (GTK_LABEL (label), FALSE);
-        gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
-	login->priv->password_entry = gnome_db_new_entry_widget (0, TRUE);
-	gtk_entry_set_visibility (GTK_ENTRY (login->priv->password_entry), FALSE);
-	gtk_table_attach (GTK_TABLE (table), login->priv->password_entry, 1, 3, 2, 3,
-			  GTK_FILL | GTK_EXPAND | GTK_SHRINK, GTK_FILL, 0, 0);
-
-	return GTK_WIDGET (login);
+	if (info) {
+		if (info->username)
+			gtk_entry_set_text (GTK_ENTRY (login->priv->username_entry), info->username);
+		if (info->password)
+			gtk_entry_set_text (GTK_ENTRY (login->priv->password_entry), info->password);
+		gda_data_source_info_free (info);
+	}
 }
 
+
 /**
  * gnome_db_login_new
  * @dsn:
@@ -305,10 +346,10 @@ gnome_db_login_construct (GnomeDbLogin *login, const gchar *dsn)
 GtkWidget *
 gnome_db_login_new (const gchar *dsn)
 {
-	GnomeDbLogin *login;
+	GtkWidget *login;
 
-	login = g_object_new (GNOME_DB_TYPE_LOGIN, NULL);
-	return gnome_db_login_construct (login, dsn);
+	login = GTK_WIDGET(g_object_new (GNOME_DB_TYPE_LOGIN, NULL));
+	return login;
 }
 
 /**
@@ -327,7 +368,7 @@ gnome_db_login_get_dsn (GnomeDbLogin *login)
 	if (login->priv->dsn_name)
 		return (const gchar *) login->priv->dsn_name;
 
-	return gnome_db_option_menu_get_selection (GTK_OPTION_MENU (login->priv->dsn_entry));
+	return gnome_db_data_source_selector_get_dsn (login->priv->dsn_entry);
 }
 
 /**
@@ -340,8 +381,20 @@ gnome_db_login_get_dsn (GnomeDbLogin *login)
 void
 gnome_db_login_set_dsn (GnomeDbLogin *login, const gchar *dsn)
 {
+	GdaDataSourceInfo *src;
 	g_return_if_fail (GNOME_DB_IS_LOGIN (login));
-	gnome_db_option_menu_set_selection (GTK_OPTION_MENU (login->priv->dsn_entry), dsn);
+
+	src = gda_config_find_data_source (dsn);
+	if (!src)
+		g_warning (_("Datasource '%s' is not declared"), dsn);
+	else {
+		if (src->username)
+			gtk_entry_set_text (GTK_ENTRY (login->priv->username_entry), src->username);
+		if (src->password)
+			gtk_entry_set_text (GTK_ENTRY (login->priv->password_entry), src->password);
+		gda_data_source_info_free (src);
+		gnome_db_data_source_selector_set_dsn (login->priv->dsn_entry, dsn);
+	}
 }
 
 /**
@@ -407,7 +460,7 @@ gnome_db_login_set_password (GnomeDbLogin *login, const gchar *password)
  * @login: A #GnomeDbLogin widget.
  * @enable: Flag to enable/disable the button.
  *
- * Enable/disable the button for allowing the user to create new data sources
+ * Enable/disable the button allowing the user to create new data sources
  * from the login widget. This button is disabled by default, and when enabled,
  * allows the user to start the database properties configuration applet.
  */
@@ -427,6 +480,24 @@ gnome_db_login_set_enable_create_button (GnomeDbLogin *login, gboolean enable)
 		gtk_widget_hide (login->priv->create_dsn_button);
 }
 
+/**
+ * gnome_db_login_get_enable_create_button
+ * @login: A #GnomeDbLogin widget.
+ *
+ * Tell if the the button allowing the user to create new data sources is shown
+ * or not. This button is disabled by default,
+ *
+ * Returns: TRUE if the button is shown
+ */
+gboolean
+gnome_db_login_get_enable_create_button (GnomeDbLogin *login)
+{
+	g_return_val_if_fail (GNOME_DB_IS_LOGIN (login), FALSE);
+
+	return ! login->priv->hiding_button;
+}
+
+
 /**
  * gnome_db_login_set_show_dsn_selector
  * @login: A #GnomeDbLogin widget.
@@ -452,3 +523,20 @@ gnome_db_login_set_show_dsn_selector (GnomeDbLogin *login, gboolean show)
 		gtk_widget_hide (login->priv->create_dsn_button);
 	}
 }
+
+/**
+ * gnome_db_login_get_show_dsn_selector
+ * @login: A #GnomeDbLogin widget.
+ *
+ * Tells if the combo box which allows to select the datasource to log into is shown
+ * or not.
+ *
+ * Returns: TRUE if the combo box is shown
+ */
+gboolean
+gnome_db_login_get_show_dsn_selector (GnomeDbLogin *login)
+{
+	g_return_val_if_fail (GNOME_DB_IS_LOGIN (login), FALSE);
+
+	return ! login->priv->hiding_dsn;
+}
