@@ -44,12 +44,12 @@ static void gnome_db_raw_grid_get_property (GObject *object,
 					    GValue *value,
 					    GParamSpec *pspec);
 
-static void gnome_db_raw_grid_initialize (GnomeDbRawGrid *grid);
 static void init_tree_view (GnomeDbRawGrid *grid);
 
 static void proxy_destroyed_cb (GdaDataProxy *proxy, GnomeDbRawGrid *grid);
 static void proxy_sample_changed_cb (GdaDataProxy *proxy, gint sample_start, gint sample_end, GnomeDbRawGrid *grid);
 static void paramlist_public_data_changed_cb (GdaParameterList *paramlist, GnomeDbRawGrid *grid);
+static void iter_row_changed_cb (GdaDataModelIter *iter, gint row, GnomeDbRawGrid *grid);
 
 /* GnomeDbDataWidget interface */
 static void            gnome_db_raw_grid_widget_init           (GnomeDbDataWidgetIface *iface);
@@ -368,6 +368,8 @@ gnome_db_raw_grid_set_property (GObject *object,
 				grid->priv->iter = gda_data_model_create_iter (GDA_DATA_MODEL (grid->priv->proxy));
 				g_signal_connect (grid->priv->iter, "public_data_changed",
 						  G_CALLBACK (paramlist_public_data_changed_cb), grid);
+				g_signal_connect (grid->priv->iter, "row_changed",
+						  G_CALLBACK (iter_row_changed_cb), grid);
 				gda_data_model_iter_invalidate_contents (grid->priv->iter);
 							
 				gtk_tree_view_set_model ((GtkTreeView *) grid, GTK_TREE_MODEL (grid->priv->store));
@@ -583,12 +585,12 @@ init_tree_view (GnomeDbRawGrid *grid)
 		}
 		else {
 			/* single direct parameter */
-			GType gda_type;
+			GType g_type;
 			gchar *plugin = NULL;
 			gchar *title;
 			
 			param = GDA_PARAMETER (GDA_PARAMETER_LIST_NODE (group->nodes->data)->param);
-			gda_type = gda_parameter_get_gda_type (param);
+			g_type = gda_parameter_get_g_type (param);
 
 			title = (gchar *) gda_object_get_name (GDA_OBJECT (param));
 			if (title && *title)
@@ -599,7 +601,7 @@ init_tree_view (GnomeDbRawGrid *grid)
 				title = g_strdup (_("No title"));
 			
 			g_object_get (G_OBJECT (param), "entry_plugin", &plugin, NULL);
-			renderer = gnome_db_util_new_cell_renderer (dict, gda_type, plugin);
+			renderer = gnome_db_util_new_cell_renderer (dict, g_type, plugin);
 			column_data->data_cell = renderer;
 			if (! column_data->data_locked)
 				column_data->data_locked = GDA_PARAMETER_LIST_NODE (group->nodes->data)->hint & 
@@ -900,16 +902,18 @@ data_cell_values_changed (GtkCellRenderer *renderer, const gchar *path,
 			params = g_slist_next (params);
 		}
 		
+#ifdef PROXY_STORE_EXTRA_VALUES
 		/* call gda_data_proxy_set_model_row_value() */
 		for (i = 0; i < group->nodes_source->shown_n_cols; i++) {
 			GValue *value;
 
 			col = group->nodes_source->shown_cols_index[i];
 			value = (GValue *) g_slist_nth_data (all_new_values, col);
-			gda_data_proxy_set_model_row_value (grid->priv->proxy, 
+			gda_data_proxy_set_model_row_value (grid->priv->proxy,
 							    group->nodes_source->data_model,
 							    proxy_row, col, value);
 		}
+#endif
 	}
 
 #ifdef debug_NO
@@ -979,6 +983,7 @@ data_cell_status_changed (GtkCellRenderer *renderer, const gchar *path, GValueAt
 			list = g_slist_next (list);
 		}
 
+#ifdef PROXY_STORE_EXTRA_VALUES
 		/* call gda_data_proxy_set_model_row_value() */
 		for (i = 0; i < group->nodes_source->shown_n_cols; i++) {
 			col = group->nodes_source->shown_cols_index[i];
@@ -1001,6 +1006,7 @@ data_cell_status_changed (GtkCellRenderer *renderer, const gchar *path, GValueAt
 				}
 			}
 		}
+#endif
 	}
 	else {
 		/* single direct parameter */
@@ -1303,8 +1309,10 @@ menu_save_as_cb (GtkWidget *widget, GnomeDbRawGrid *grid)
 	GtkWidget *label;
 	GtkWidget *filename;
 	GtkWidget *types;
-	GtkWidget *hbox;
+	GtkWidget *hbox, *table, *check;
 	char *str;
+	GtkTreeSelection *sel;
+	gint selrows;
 
 	/* create dialog box */
 	dialog = gtk_dialog_new_with_buttons (_("Saving Data"),
@@ -1343,7 +1351,7 @@ menu_save_as_cb (GtkWidget *widget, GnomeDbRawGrid *grid)
 	g_object_set_data (G_OBJECT (dialog), "filename", filename);
 	gtk_box_pack_start (GTK_BOX (hbox), filename, TRUE, TRUE, 0);
 
-	str = g_strdup_printf ("<b>%s:</b>", _("File type"));
+	str = g_strdup_printf ("<b>%s:</b>", _("Details")); 
         label = gtk_label_new ("");
         gtk_label_set_markup (GTK_LABEL (label), str);
         gtk_misc_set_alignment (GTK_MISC (label), 0., -1);
@@ -1356,9 +1364,40 @@ menu_save_as_cb (GtkWidget *widget, GnomeDbRawGrid *grid)
         label = gtk_label_new ("    ");
         gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
         gtk_widget_show (label);
+	
+	table = gtk_table_new (2, 2, FALSE);
+	gtk_table_set_row_spacings (GTK_TABLE (table), 5);
+	gtk_table_set_col_spacings (GTK_TABLE (table), 5);
+	gtk_box_pack_start (GTK_BOX (hbox), table, TRUE, TRUE, 0);
+	gtk_widget_show (table);
+
+	label = gtk_label_new (_("Limit to selection?"));
+	gtk_misc_set_alignment (GTK_MISC (label), 0., -1);
+	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, GTK_FILL, 0, 0, 0);
+	gtk_widget_show (label);
+
+	sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (grid));
+	selrows = gtk_tree_selection_count_selected_rows (sel);
+	if (selrows <= 0)
+		gtk_widget_set_sensitive (label, FALSE);
+
+	check = gtk_check_button_new ();
+	gtk_table_attach_defaults (GTK_TABLE (table), check, 1, 2, 0, 1);
+	gtk_widget_show (check);
+	if (selrows <= 0)
+		gtk_widget_set_sensitive (check, FALSE);
+	else
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), TRUE);
+	g_object_set_data (G_OBJECT (dialog), "sel_only", check);
+
+	label = gtk_label_new (_("File type:"));
+	gtk_misc_set_alignment (GTK_MISC (label), 0., -1);
+	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, GTK_FILL, 0, 0, 0);
+	gtk_widget_show (label);
 
 	types = gtk_combo_box_new_text ();
-	gtk_box_pack_start (GTK_BOX (hbox), types, TRUE, TRUE, 0);
+	gtk_table_attach_defaults (GTK_TABLE (table), types, 1, 2, 1, 2);
+	gtk_widget_show (label);
 	g_object_set_data (G_OBJECT (dialog), "types", types);
 
 	gtk_combo_box_append_text (GTK_COMBO_BOX (types), _("Tab-delimited"));
@@ -1379,17 +1418,21 @@ save_as_response_cb (GtkDialog *dialog, guint response_id, GnomeDbRawGrid *grid)
 	GtkWidget *types;
 	gint export_type;
 	GtkWidget *filename;
+	gboolean selection_only = FALSE;
 	
 	if (response_id == GTK_RESPONSE_OK) {
 		gchar *body;
 		gchar *path;
 		GList *columns, *list;
 		gint *cols, nb_cols;
+		gint *rows = NULL, nb_rows = 0;
 		GdaParameter *param;
 		GdaParameterList *paramlist;
 
 		types = g_object_get_data (G_OBJECT (dialog), "types");
 		filename = g_object_get_data (G_OBJECT (dialog), "filename");
+		selection_only = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON 
+							       (g_object_get_data (G_OBJECT (dialog), "sel_only")));
 
 		/* output columns computation */
 		columns = gtk_tree_view_get_columns (GTK_TREE_VIEW (grid));
@@ -1413,6 +1456,26 @@ save_as_response_cb (GtkDialog *dialog, guint response_id, GnomeDbRawGrid *grid)
 			list = g_list_next (list);
 		}
 		g_list_free (columns);
+
+		/* output rows computation */
+		if (selection_only) {
+			GtkTreeSelection *selection;
+			GList *sel_rows, *list;
+			gint pos;
+
+			selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (grid));
+			sel_rows = gtk_tree_selection_get_selected_rows (selection, NULL);
+
+			nb_rows = g_list_length (sel_rows);
+			rows = g_new0 (gint, nb_rows);
+
+			for (pos = 0, list = sel_rows; list; list = list->next, pos++) {
+				gint *ind = gtk_tree_path_get_indices ((GtkTreePath*) list->data);
+				rows [pos] = *ind;
+				gtk_tree_path_free ((GtkTreePath*) list->data);
+			}
+			g_list_free (sel_rows);
+		}
 		
 		/* Actual ouput computations */
 		export_type = gtk_combo_box_get_active (GTK_COMBO_BOX (types));
@@ -1425,7 +1488,7 @@ save_as_response_cb (GtkDialog *dialog, guint response_id, GnomeDbRawGrid *grid)
 			g_object_unref (param);
 			body = gda_data_model_export_to_string (GDA_DATA_MODEL (grid->priv->data_model), 
 								GDA_DATA_MODEL_IO_TEXT_SEPARATED, 
-								cols, nb_cols, paramlist);
+								cols, nb_cols, rows, nb_rows, paramlist);
 			g_object_unref (paramlist);
 			break;
 		case 1:
@@ -1435,7 +1498,7 @@ save_as_response_cb (GtkDialog *dialog, guint response_id, GnomeDbRawGrid *grid)
 			g_object_unref (param);
 			body = gda_data_model_export_to_string (GDA_DATA_MODEL (grid->priv->data_model), 
 								GDA_DATA_MODEL_IO_TEXT_SEPARATED, 
-								cols, nb_cols, paramlist);
+								cols, nb_cols, rows, nb_rows, paramlist);
 			g_object_unref (paramlist);
 			break;
 		case 2:
@@ -1448,7 +1511,7 @@ save_as_response_cb (GtkDialog *dialog, guint response_id, GnomeDbRawGrid *grid)
 			g_object_unref (param);
 			body = gda_data_model_export_to_string (GDA_DATA_MODEL (grid->priv->data_model), 
 								GDA_DATA_MODEL_IO_DATA_ARRAY_XML, 
-								cols, nb_cols, paramlist);
+								cols, nb_cols, rows, nb_rows, paramlist);
 			g_object_unref (paramlist);
 			break;
 		default:
@@ -1456,6 +1519,8 @@ save_as_response_cb (GtkDialog *dialog, guint response_id, GnomeDbRawGrid *grid)
 			break;
 		}
 		g_free (cols);
+		if (rows)
+			g_free (rows);
 
 		/* saving */
 		if (body) {
@@ -1554,6 +1619,10 @@ tree_view_selection_changed_cb (GtkTreeSelection *selection, GnomeDbRawGrid *gri
 	gboolean row_selected = FALSE;
 	gint has_selection;
 	
+	/* block the GdaDataModelIter' "changed" signal */
+	g_signal_handlers_block_by_func (grid->priv->iter,
+					 G_CALLBACK (iter_row_changed_cb), grid);
+
 	if (gtk_tree_selection_get_mode (selection) == GTK_SELECTION_MULTIPLE) {
 		has_selection = gtk_tree_selection_count_selected_rows (selection);
 		if (has_selection == 1) {
@@ -1588,6 +1657,10 @@ tree_view_selection_changed_cb (GtkTreeSelection *selection, GnomeDbRawGrid *gri
 #ifdef debug_signal
 	g_print ("<< 'SELECTION_CHANGED' from %s %p\n", G_OBJECT_TYPE_NAME (grid), grid);
 #endif
+
+	/* unblock the GdaDataModelIter' "changed" signal */
+	g_signal_handlers_unblock_by_func (grid->priv->iter,
+					   G_CALLBACK (iter_row_changed_cb), grid);
 }
 
 static ColumnData *
@@ -1805,6 +1878,30 @@ paramlist_public_data_changed_cb (GdaParameterList *paramlist, GnomeDbRawGrid *g
 	init_tree_view (grid);
 }
 
+static void
+iter_row_changed_cb (GdaDataModelIter *iter, gint row, GnomeDbRawGrid *grid)
+{
+	GtkTreeSelection *selection;
+	GtkTreePath *path;
+
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (grid));
+	
+	if (row >= 0) {
+		GtkSelectionMode mode;
+
+		mode = gtk_tree_selection_get_mode (selection);
+		if (mode != GTK_SELECTION_SINGLE)
+			gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
+		path = gtk_tree_path_new_from_indices (row, -1);
+		gtk_tree_selection_select_path (selection, path);
+		gtk_tree_path_free (path);
+		if (mode != GTK_SELECTION_SINGLE)
+			gtk_tree_selection_set_mode (selection, mode);
+	}
+	else
+		gtk_tree_selection_unselect_all (selection);
+}
+
 static void
 proxy_sample_changed_cb (GdaDataProxy *proxy, gint sample_start, gint sample_end, GnomeDbRawGrid *grid)
 {
@@ -1863,6 +1960,8 @@ gnome_db_raw_grid_clean (GnomeDbRawGrid *grid)
 	if (grid->priv->iter) {
 		g_signal_handlers_disconnect_by_func (grid->priv->iter,
 						      G_CALLBACK (paramlist_public_data_changed_cb), grid);
+		g_signal_handlers_disconnect_by_func (grid->priv->iter,
+						      G_CALLBACK (iter_row_changed_cb), grid);
 		g_object_unref (grid->priv->iter);
 		grid->priv->iter = NULL;
 	}
