@@ -1,6 +1,6 @@
 /* gnome-db-selector.c
  *
- * Copyright (C) 2002 - 2005 Vivien Malerba
+ * Copyright (C) 2002 - 2006 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,13 +19,11 @@
  */
 
 #include <string.h>
-#include <libgnomedb/libgnomedb.h>
 #include "gnome-db-selector.h"
 #include "marshal.h"
 
-#include "gnome-db-table.h"
-#include "gnome-db-query.h"
-
+#include <libgda/libgda.h>
+#include <glib/gi18n-lib.h>
 #include "gnome-db-selector-private.h"
 
 #include "sel-data-types.h"
@@ -36,7 +34,6 @@
 #include "sel-queries.h"
 #include "sel-onequery.h"
 #include "sel-graphs.h"
-#include "sel-forms.h"
 
 static void gnome_db_selector_class_init (GnomeDbSelectorClass * class);
 static void gnome_db_selector_init (GnomeDbSelector * wid);
@@ -124,7 +121,7 @@ static gboolean gnome_db_selector_initialize (GnomeDbSelector *mgsel, GObject *r
 static void object_weak_notify (GnomeDbSelector *mgsel, GObject *obj);
 /**
  * gnome_db_selector_new
- * @dict: a #GnomeDbDict object
+ * @dict: a #GdaDict object
  * @ref_object: a #GObject object, or %NULL if none is required
  * @mode: an OR'ed value of the possible items to display in the widget
  * @columns: an OR'ed value describing which columns will be displayed
@@ -144,12 +141,12 @@ static void object_weak_notify (GnomeDbSelector *mgsel, GObject *obj);
  * Returns: the new widget
  */
 GtkWidget *
-gnome_db_selector_new (GnomeDbDict *dict, GObject *ref_object, gulong mode, gulong columns)
+gnome_db_selector_new (GdaDict *dict, GObject *ref_object, gulong mode, gulong columns)
 {
 	GObject    *obj;
 	GnomeDbSelector *mgsel;
 
-	g_return_val_if_fail (!dict || IS_GNOME_DB_DICT (dict), NULL);
+	g_return_val_if_fail (!dict || GDA_IS_DICT (dict), NULL);
 		
 	obj = g_object_new (GNOME_DB_TYPE_SELECTOR, NULL);
 	mgsel = GNOME_DB_SELECTOR (obj);
@@ -191,7 +188,7 @@ object_weak_notify (GnomeDbSelector *mgsel, GObject *obj)
 		gtk_tree_store_clear (GTK_TREE_STORE (mgsel->priv->model));
 	
 	if (obj == (GObject*) mgsel->priv->dict)
-		/* Tell that we don't need to weak unref the GnomeDbDict */
+		/* Tell that we don't need to weak unref the GdaDict */
 		mgsel->priv->dict = NULL;
 	if (obj == (GObject *) mgsel->priv->ref_object)
 		/* Tell that we don't need to weak unref the Ref object */
@@ -234,7 +231,7 @@ gnome_db_selector_dispose (GObject *object)
 		g_slist_free (mgsel->priv->modules);
 		mgsel->priv->modules = NULL;
 
-		/* Weak unref the GnomeDbDict if necessary */
+		/* Weak unref the GdaDict if necessary */
 		if (mgsel->priv->dict)
 			g_object_weak_unref (G_OBJECT (mgsel->priv->dict),
 					     (GWeakNotify) object_weak_notify, mgsel);
@@ -278,7 +275,7 @@ gnome_db_selector_initialize (GnomeDbSelector *mgsel, GObject *ref_object, gbool
 	 */
 	if (!(mgsel->priv->mode & GNOME_DB_SELECTOR_TABLES) && 
 	    (mgsel->priv->mode & GNOME_DB_SELECTOR_FIELDS) && 
-	    (!ref_object || !IS_GNOME_DB_TABLE (ref_object))) {
+	    (!ref_object || !GDA_IS_DICT_TABLE (ref_object))) {
 		return FALSE;
 	}
 	
@@ -289,7 +286,7 @@ gnome_db_selector_initialize (GnomeDbSelector *mgsel, GObject *ref_object, gbool
 	     (mgsel->priv->mode & GNOME_DB_SELECTOR_QVIS_FIELDS) ||
 	     (mgsel->priv->mode & GNOME_DB_SELECTOR_QALL_FIELDS) ||
 	     (mgsel->priv->mode & GNOME_DB_SELECTOR_SUB_QUERIES)) && 
-	    (!ref_object || !IS_GNOME_DB_QUERY (ref_object))) {
+	    (!ref_object || !GDA_IS_QUERY (ref_object))) {
 		return FALSE;
 	}
 
@@ -359,7 +356,8 @@ gnome_db_selector_initialize (GnomeDbSelector *mgsel, GObject *ref_object, gbool
 							    GDK_TYPE_PIXBUF,  /* PIXBUF_COLUMN */
 							    G_TYPE_INT,       /* CONTENTS_COLUMN */
 							    G_TYPE_POINTER,   /* SUB_MODULE_COLUMN */
-							    G_TYPE_BOOLEAN)); /* ERROR_COLUMN marker column */
+							    G_TYPE_BOOLEAN,   /* ERROR_COLUMN */
+							    G_TYPE_STRING));  /* BG_COLOR_COLUMN */
 		mgsel->priv->model = model;
 
 		/* 'top level' modules selection (not using ref_object) */
@@ -393,11 +391,6 @@ gnome_db_selector_initialize (GnomeDbSelector *mgsel, GObject *ref_object, gbool
 			mgsel->priv->modules = g_slist_append (mgsel->priv->modules, module);
 		}
 
-		if (mgsel->priv->mode & GNOME_DB_SELECTOR_FORMS) {
-			Module *module = sel_module_forms_new (mgsel, need_header, NULL, NULL);
-			mgsel->priv->modules = g_slist_append (mgsel->priv->modules, module);
-		}
-
 		if (!mgsel->priv->modules) {
 			/* 'low level' modules selection (making use of ref_object) */
 			if (mgsel->priv->mode & GNOME_DB_SELECTOR_FIELDS) {
@@ -536,17 +529,21 @@ gnome_db_selector_initialize (GnomeDbSelector *mgsel, GObject *ref_object, gbool
 						      "strikethrough-set", FALSE, NULL);
 					column = gtk_tree_view_column_new_with_attributes (str, renderer, 
 											   "text", i, 
-											   "strikethrough-set", ERROR_COLUMN, NULL);
+											   "strikethrough-set", ERROR_COLUMN, 
+											   "cell-background", BG_COLOR_COLUMN,
+											   NULL);
 					break;
 				case G_TYPE_BOOLEAN:
 					renderer = gtk_cell_renderer_toggle_new ();
 					column = gtk_tree_view_column_new_with_attributes (str, renderer, 
 											   "active", i, 
 											   "visible", i+1, 
+											   "cell-background", BG_COLOR_COLUMN,
 											   NULL);
 					break;
 				}
 				
+				g_object_set (G_OBJECT (renderer), "cell-background-set", TRUE, NULL);
 				gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
 			}
 		}
@@ -580,17 +577,26 @@ tree_value_set_func (GtkTreeViewColumn *tree_column,
                      gpointer user_data)
 {
         GdkPixbuf *pixbuf;
-        gchar *text;
+        gchar *text, *bg_color;
         GList *renderers;
 	gboolean error;
 
-        gtk_tree_model_get (model, iter, PIXBUF_COLUMN, &pixbuf, NAME_COLUMN, &text, ERROR_COLUMN, &error, -1);
+        gtk_tree_model_get (model, iter, PIXBUF_COLUMN, &pixbuf, 
+			    NAME_COLUMN, &text, 
+			    BG_COLOR_COLUMN, &bg_color, 
+			    ERROR_COLUMN, &error, -1);
         renderers = gtk_tree_view_column_get_cell_renderers (tree_column);
         cell = renderers->data;
-        g_object_set (G_OBJECT (cell), "pixbuf", pixbuf, NULL);
+        g_object_set (G_OBJECT (cell), "pixbuf", pixbuf, 
+		      "cell-background", bg_color, 
+		      "cell-background-set", bg_color ? TRUE : FALSE, NULL);
         cell = renderers->next->data;
-        g_object_set (G_OBJECT (cell), "text", text, "strikethrough-set", error, NULL);
+        g_object_set (G_OBJECT (cell), "text", text, 
+		      "strikethrough-set", error, 
+		      "cell-background", bg_color,
+		      "cell-background-set", bg_color ? TRUE : FALSE, NULL);
         g_free (text);
+        g_free (bg_color);
         g_list_free (renderers);
 }
 
@@ -808,6 +814,38 @@ gnome_db_selector_set_column_label (GnomeDbSelector *mgsel, guint column, const
 	gtk_tree_view_column_set_title (tcolumn, label);
 }
 
+/**
+ * gnome_db_selector_expand_all
+ * @mgsel:
+ *
+ * Recursively expands all nodes in the @mgsel widget.
+ */
+void
+gnome_db_selector_expand_all (GnomeDbSelector *mgsel)
+{
+	g_return_if_fail (mgsel && IS_GNOME_DB_SELECTOR (mgsel));
+	g_return_if_fail (mgsel->priv);
+
+	if (mgsel->priv->treeview)
+		gtk_tree_view_expand_all (mgsel->priv->treeview);
+}
+
+/**
+ * gnome_db_selector_collapse_all
+ * @mgsel:
+ *
+ * Recursively collapse all nodes in the @mgsel widget.
+ */
+void
+gnome_db_selector_collapse_all (GnomeDbSelector *mgsel)
+{
+	g_return_if_fail (mgsel && IS_GNOME_DB_SELECTOR (mgsel));
+	g_return_if_fail (mgsel->priv);
+
+	if (mgsel->priv->treeview)
+		gtk_tree_view_collapse_all (mgsel->priv->treeview);
+}
+
 
 /**
  * gnome_db_selector_get_selected_object
@@ -995,8 +1033,8 @@ set_iter_position (GtkTreeModel *model, GSList *obj_list, gpointer object, gpoin
 			gtk_tree_model_get (model, iter, OBJ_COLUMN, &obj, -1);
 			if (obj != object) {
 				g_warning ("Obj found %p (%s=%s) != object requested %p (%s=%s)\n", 
-					   obj, G_OBJECT_TYPE_NAME (obj), gnome_db_base_get_name (GNOME_DB_BASE (obj)),
-					   object, G_OBJECT_TYPE_NAME (object), gnome_db_base_get_name (GNOME_DB_BASE (object)));
+					   obj, G_OBJECT_TYPE_NAME (obj), gda_object_get_name (GDA_OBJECT (obj)),
+					   object, G_OBJECT_TYPE_NAME (object), gda_object_get_name (GDA_OBJECT (object)));
 				error = TRUE;
 			}
 		}
@@ -1021,7 +1059,7 @@ model_store_data (Module *module, GtkTreeIter *iter)
 
 
 /*
- * Generic module model where objects are grouped by their gnome_db_base_get_name() value
+ * Generic module model where objects are grouped by their gda_object_get_name() value
  * The text which gets displayed
  */
 
@@ -1042,13 +1080,13 @@ name_group_init_model_fill (Module *module, GtkTreeModel *model)
 		gchar *str1;
 		const gchar *str2, *str3;
 
-		if (strcmp (current_name, gnome_db_base_get_name (GNOME_DB_BASE (ptl->data)))) {
+		if (strcmp (current_name, gda_object_get_name (GDA_OBJECT (ptl->data)))) {
 			GSList *next_obj = g_slist_next (ptl);
 			/* function with a different name */
-			current_name = gnome_db_base_get_name (GNOME_DB_BASE (ptl->data));
+			current_name = gda_object_get_name (GDA_OBJECT (ptl->data));
 			
 			if (next_obj &&
-			    !strcmp (gnome_db_base_get_name (GNOME_DB_BASE (next_obj->data)), current_name)) {
+			    !strcmp (gda_object_get_name (GDA_OBJECT (next_obj->data)), current_name)) {
 				/* a new category is needed */
 				gtk_tree_store_append (GTK_TREE_STORE (model), &cat_iter, module->iter);
 				gtk_tree_store_set (GTK_TREE_STORE (model), &cat_iter, 
@@ -1067,8 +1105,8 @@ name_group_init_model_fill (Module *module, GtkTreeModel *model)
 			gtk_tree_store_append (GTK_TREE_STORE (model), &iter, module->iter);
 
 		str1 = GROUP_DATA (module)->get_extended_name (G_OBJECT (ptl->data));
-		str2 = gnome_db_base_get_owner (GNOME_DB_BASE (ptl->data));
-		str3 = gnome_db_base_get_description (GNOME_DB_BASE (ptl->data));
+		str2 = gda_object_get_owner (GDA_OBJECT (ptl->data));
+		str3 = gda_object_get_description (GDA_OBJECT (ptl->data));
 		gtk_tree_store_set (GTK_TREE_STORE (model), &iter, 
 				    NAME_COLUMN, str1,
 				    OWNER_COLUMN, str2, 
@@ -1101,12 +1139,12 @@ name_group_init_model_fill (Module *module, GtkTreeModel *model)
 	GROUP_DATA (module)->iter_obj = NULL;
 	GROUP_DATA (module)->iter = NULL;
 
-	/* manager "nullified" connect if possible, or otherwise only weak ref */
-	if (g_signal_lookup ("nullified", G_OBJECT_TYPE (GROUP_DATA (module)->manager)))
-		GROUP_DATA (module)->manager_nullified_sig_id = 
-			gnome_db_base_connect_nullify (
+	/* manager "destroyed" connect if possible, or otherwise only weak ref */
+	if (g_signal_lookup ("destroyed", G_OBJECT_TYPE (GROUP_DATA (module)->manager)))
+		GROUP_DATA (module)->manager_destroyed_sig_id = 
+			gda_object_connect_destroy (
 				GROUP_DATA (module)->manager, 
-				G_CALLBACK (name_group_manager_nullified_cb), module);
+				G_CALLBACK (name_group_manager_destroyed_cb), module);
 	else {
 		g_object_weak_ref (G_OBJECT (GROUP_DATA (module)->manager),
 				   (GWeakNotify) name_group_manager_weak_notify, module);
@@ -1118,14 +1156,14 @@ name_group_init_model_fill (Module *module, GtkTreeModel *model)
  * Remove the module
  */
 void
-name_group_manager_nullified_cb (GObject *manager_obj, Module *module)
+name_group_manager_destroyed_cb (GObject *manager_obj, Module *module)
 {
 	GtkTreeModel *model;
 	
 	model = module->selector->priv->model;
 	g_signal_handler_disconnect (G_OBJECT (GROUP_DATA (module)->manager),
-				     GROUP_DATA (module)->manager_nullified_sig_id);
-	GROUP_DATA (module)->manager_nullified_sig_id = 0;
+				     GROUP_DATA (module)->manager_destroyed_sig_id);
+	GROUP_DATA (module)->manager_destroyed_sig_id = 0;
 	
 	if (module->iter) {
 		gtk_tree_store_remove (GTK_TREE_STORE (model), module->iter);
@@ -1180,7 +1218,7 @@ name_group_obj_added_cb (GObject *manager_obj, GObject *added_obj, Module *modul
 	gint objpos;
 
 	/* g_print ("ADDING obj %p of type %s ", added_obj, G_OBJECT_TYPE_NAME (added_obj)); */
-	/* 	g_print (" NAME: %s\n", gnome_db_base_get_name (GNOME_DB_BASE (added_obj))); */
+	/* 	g_print (" NAME: %s\n", gda_object_get_name (GDA_OBJECT (added_obj))); */
 	
 
 	/* module's objects list sync. */
@@ -1301,21 +1339,20 @@ name_group_do_add_obj (Module *module, GObject *added_obj)
 	const gchar *objname;
 	gboolean insert_ok = FALSE;
 
-	objname = gnome_db_base_get_name (GNOME_DB_BASE (added_obj));
+	objname = gda_object_get_name (GDA_OBJECT (added_obj));
 	model = module->selector->priv->model;
 
-
 	/* finding the siblings of the new object */
 	objpos = g_slist_index (GROUP_DATA (module)->objects, added_obj);
 	if (objpos > 0) {
 		sibling_before = g_slist_nth_data (GROUP_DATA (module)->objects, objpos - 1);
-		if (strcmp (gnome_db_base_get_name (GNOME_DB_BASE (sibling_before)), objname))
+		if (strcmp (gda_object_get_name (GDA_OBJECT (sibling_before)), objname))
 			sibling_before = NULL;
 	}
 
 	sibling_after = g_slist_nth_data (GROUP_DATA (module)->objects, objpos + 1);
 	if (sibling_after) {
-		if (strcmp (gnome_db_base_get_name (GNOME_DB_BASE (sibling_after)), objname))
+		if (strcmp (gda_object_get_name (GDA_OBJECT (sibling_after)), objname))
 			sibling_after = NULL;
 	}
 
@@ -1426,15 +1463,15 @@ name_group_do_add_obj (Module *module, GObject *added_obj)
 					tmpiter = cat_iter;
 					
 					gtk_tree_store_set (GTK_TREE_STORE (model), &cat_iter, NAME_COLUMN, 
-							    gnome_db_base_get_name (GNOME_DB_BASE (added_obj)), 
+							    gda_object_get_name (GDA_OBJECT (added_obj)), 
 							    CONTENTS_COLUMN, CONTENTS_GROUP_CATEGORY, 
 							    SUB_MODULE_COLUMN, NULL, -1);
 					gtk_tree_store_remove (GTK_TREE_STORE (model), &iter);
 					gtk_tree_store_append (GTK_TREE_STORE (model), &iter, &cat_iter);
 					
 					str1 = GROUP_DATA (module)->get_extended_name (sibling_obj);
-					str2 = gnome_db_base_get_owner (GNOME_DB_BASE (sibling_obj));
-					str3 = gnome_db_base_get_description (GNOME_DB_BASE (sibling_obj));
+					str2 = gda_object_get_owner (GDA_OBJECT (sibling_obj));
+					str3 = gda_object_get_description (GDA_OBJECT (sibling_obj));
 					gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
 							    NAME_COLUMN, str1, 
 							    OWNER_COLUMN, str2, 
@@ -1469,8 +1506,8 @@ name_group_do_add_obj (Module *module, GObject *added_obj)
 		const gchar *str2, *str3;
 
 		str1 = GROUP_DATA (module)->get_extended_name (added_obj);
-		str2 = gnome_db_base_get_owner (GNOME_DB_BASE (added_obj));
-		str3 = gnome_db_base_get_description (GNOME_DB_BASE (added_obj));
+		str2 = gda_object_get_owner (GDA_OBJECT (added_obj));
+		str3 = gda_object_get_description (GDA_OBJECT (added_obj));
 		gtk_tree_store_set (GTK_TREE_STORE (model), &iter, 
 				    NAME_COLUMN, str1,
 				    OWNER_COLUMN, str2,
@@ -1568,7 +1605,7 @@ name_group_do_remove_obj (Module *module, GObject *removed_obj)
 	else
 		g_warning ("Can't find right GtkTreeIter for object %p (%s)!",
 			   removed_obj, 
-			   removed_obj ? gnome_db_base_get_name (GNOME_DB_BASE (removed_obj)) : "NULL");
+			   removed_obj ? gda_object_get_name (GDA_OBJECT (removed_obj)) : "NULL");
 }
 
 /*
@@ -1592,8 +1629,8 @@ name_group_do_update_obj (Module *module, GObject *upd_obj)
 		const gchar *str2, *str3;
 
 		str1 = GROUP_DATA (module)->get_extended_name (upd_obj);
-		str2 = gnome_db_base_get_owner (GNOME_DB_BASE (upd_obj));
-		str3 = gnome_db_base_get_description (GNOME_DB_BASE (upd_obj));
+		str2 = gda_object_get_owner (GDA_OBJECT (upd_obj));
+		str3 = gda_object_get_description (GDA_OBJECT (upd_obj));
 		gtk_tree_store_set (GTK_TREE_STORE (model), &iter, 
 				    NAME_COLUMN, str1,
 				    OWNER_COLUMN, str2,
@@ -1639,11 +1676,11 @@ name_group_free_mod_data (Module *module)
 		GROUP_DATA (module)->manager_weak_refed = FALSE;
 	}
 
-	/* manager "nullified" signal disconnect */
-	if (GROUP_DATA (module)->manager_nullified_sig_id) {
+	/* manager "destroyed" signal disconnect */
+	if (GROUP_DATA (module)->manager_destroyed_sig_id) {
 		g_signal_handler_disconnect (G_OBJECT (GROUP_DATA (module)->manager),
-					     GROUP_DATA (module)->manager_nullified_sig_id);
-		GROUP_DATA (module)->manager_nullified_sig_id = 0;
+					     GROUP_DATA (module)->manager_destroyed_sig_id);
+		GROUP_DATA (module)->manager_destroyed_sig_id = 0;
 	}
 }
 
@@ -1686,9 +1723,9 @@ flat_init_model_fill (Module *module, GtkTreeModel *model)
 		if (! pixbuf)
 			pixbuf = FLAT_DATA (module)->fallback_obj_pixbuf;
 		
-		str1 = gnome_db_base_get_name (GNOME_DB_BASE (ptl->data));
-		str2 = gnome_db_base_get_owner (GNOME_DB_BASE (ptl->data));
-		str3 = gnome_db_base_get_description (GNOME_DB_BASE (ptl->data));
+		str1 = gda_object_get_name (GDA_OBJECT (ptl->data));
+		str2 = gda_object_get_owner (GDA_OBJECT (ptl->data));
+		str3 = gda_object_get_description (GDA_OBJECT (ptl->data));
 		gtk_tree_store_set (GTK_TREE_STORE (model), &iter, 
 				    NAME_COLUMN, str1,
 				    OWNER_COLUMN, str2, 
@@ -1863,9 +1900,9 @@ flat_do_add_obj (Module *module, GObject *added_obj)
 		pixbuf = FLAT_DATA (module)->fallback_obj_pixbuf;
 
 	gtk_tree_store_insert (GTK_TREE_STORE (model), &iter, module->iter, objpos);
-	str1 = gnome_db_base_get_name (GNOME_DB_BASE (added_obj));
-	str2 = gnome_db_base_get_owner (GNOME_DB_BASE (added_obj));
-	str3 = gnome_db_base_get_description (GNOME_DB_BASE (added_obj));
+	str1 = gda_object_get_name (GDA_OBJECT (added_obj));
+	str2 = gda_object_get_owner (GDA_OBJECT (added_obj));
+	str3 = gda_object_get_description (GDA_OBJECT (added_obj));
 	gtk_tree_store_set (GTK_TREE_STORE (model), &iter, 
 			    NAME_COLUMN, str1,
 			    OWNER_COLUMN, str2,
@@ -1926,7 +1963,7 @@ flat_do_remove_obj (Module *module, GObject *removed_obj)
 	else
 		g_warning ("Can't find right GtkTreeIter for object %p (%s) at position %d!",
 			   removed_obj, 
-			   removed_obj ? gnome_db_base_get_name (GNOME_DB_BASE (removed_obj)) : "NULL", objpos);
+			   removed_obj ? gda_object_get_name (GDA_OBJECT (removed_obj)) : "NULL", objpos);
 }
 
 /*
@@ -1957,9 +1994,9 @@ flat_do_update_obj (Module *module, GObject *upd_obj)
 		if (! pixbuf)
 			pixbuf = FLAT_DATA (module)->fallback_obj_pixbuf;
 
-		str1 = gnome_db_base_get_name (GNOME_DB_BASE (upd_obj));
-		str2 = gnome_db_base_get_owner (GNOME_DB_BASE (upd_obj));
-		str3 = gnome_db_base_get_description (GNOME_DB_BASE (upd_obj));
+		str1 = gda_object_get_name (GDA_OBJECT (upd_obj));
+		str2 = gda_object_get_owner (GDA_OBJECT (upd_obj));
+		str3 = gda_object_get_description (GDA_OBJECT (upd_obj));
 		gtk_tree_store_set (GTK_TREE_STORE (model), &iter, 
 				    NAME_COLUMN, str1,
 				    OWNER_COLUMN, str2,
@@ -1972,7 +2009,7 @@ flat_do_update_obj (Module *module, GObject *upd_obj)
 	else
 		g_warning ("Can't find right GtkTreeIter for object %p (%s) at position %d!",
 			   upd_obj, 
-			   upd_obj ? gnome_db_base_get_name (GNOME_DB_BASE (upd_obj)) : "NULL", objpos);
+			   upd_obj ? gda_object_get_name (GDA_OBJECT (upd_obj)) : "NULL", objpos);
 }
 
 
