@@ -221,17 +221,18 @@ utility_proxy_compute_attributes_for_group (GdaParameterListGroup *group, GnomeD
         offset = gda_data_proxy_get_proxied_model_n_cols (proxy);
 
         /* list the values in proxy_model for each param in GDA_PARAMETER_LIST_NODE (group->nodes->data)->params */
-        list = group->nodes;
-        while (list) {
+	attributes = 0;
+        for (list = group->nodes; list; list = list->next) {
 		col = gda_data_model_iter_get_column_for_param (model_iter, GDA_PARAMETER_LIST_NODE (list->data)->param);
                 gtk_tree_model_get (GTK_TREE_MODEL (store), tree_iter,
                                     DATA_STORE_COL_TO_DELETE, &local_to_del,
                                     offset + col, &localattr, -1);
-                attributes &= localattr;
+		if (list == group->nodes)
+			attributes = localattr;
+		else
+			attributes &= localattr;
                 to_del = to_del && local_to_del;
 		new_row = (row < 0) ? TRUE : FALSE;
-
-                list = g_slist_next (list);
         }
 
 	if (to_be_deleted)
@@ -248,7 +249,7 @@ utility_proxy_compute_attributes_for_group (GdaParameterListGroup *group, GnomeD
  * If @model_values is TRUE, then the values in the returned list are the values of the
  * @group->nodes_source->data_model, at the @group->nodes_source->shown_cols_index columns
  *
- * For both performances resaons and because the contents of the @group->nodes_source->data_model
+ * For both performances reasons and because the contents of the @group->nodes_source->data_model
  * may change, this function uses the gda_data_proxy_get_model_row_value() function.
  *
  * The returned list must be freed by the caller, but the values in the list must not be modified or
@@ -292,6 +293,7 @@ utility_proxy_compute_values_for_group (GdaParameterListGroup *group, GnomeDbDat
 		source = group->nodes_source;
 		for (i = 0 ; (i < source->shown_n_cols)  && !ret_null; i++) {
 			col = source->shown_cols_index[i];
+#ifdef PROXY_STORE_EXTRA_VALUES
 			if (!slow_way) {
 				value = gda_data_proxy_get_model_row_value (proxy, source->data_model, proxy_row, col);
 				if (value)
@@ -303,6 +305,8 @@ utility_proxy_compute_values_for_group (GdaParameterListGroup *group, GnomeDbDat
 						retval = g_list_append (retval, NULL);
 				}
 			}
+#endif
+			slow_way = TRUE;
 			
 			if (slow_way) {
 				/* may be a bit slow: try to find the values in the model, it's better if the user
@@ -335,6 +339,7 @@ utility_proxy_compute_values_for_group (GdaParameterListGroup *group, GnomeDbDat
 					retval = g_list_append (retval, (GValue *) value);
 				}
 				else {
+#ifdef DEBUG_WARNING
 					g_warning ("Could not find requested value in restricting data model");
 					g_print ("Requested: ");
 					list = key_values;
@@ -354,6 +359,7 @@ utility_proxy_compute_values_for_group (GdaParameterListGroup *group, GnomeDbDat
 					}
 					g_print (" in data model: \n");
 					gda_data_model_dump (source->data_model, stdout);
+#endif
 					ret_null = TRUE;
 				}
 				g_slist_free (key_values);
