@@ -1,7 +1,9 @@
 #include <glib/gi18n-lib.h>
 #include <libgnomedb/libgnomedb.h>
 #include <gtk/gtk.h>
-#include <linux/unistd.h>
+#include <unistd.h>
+#include <string.h>
+#include <libgda/graph/gda-dict-reg-graphs.h>
 
 #define SQL_QUERY_ROWS "SELECT c.name, o.creation_date FROM orders o INNER JOIN customers c ON (c.id=o.customer) ORDER BY c.name, o.creation_date"
 #define SQL_QUERY_COLS "SELECT p.name, p.price FROM products p ORDER BY p.name, p.price"
@@ -116,6 +118,11 @@ static gboolean   forms_grids_test_signal    (MainConfig *config, GtkWidget *tes
 static GtkWidget *cust_forms_grids_test_make (MainConfig *config);
 static gboolean   cust_forms_grids_test_signal    (MainConfig *config, GtkWidget *test_widget);
 
+static GtkWidget *import_test_make   (MainConfig *config);
+
+static GtkWidget *server_op_test_make  (MainConfig *config);
+static void       server_op_test_clean (MainConfig *config, GtkWidget *test_widget);
+static gboolean   server_op_test_signal (MainConfig *config, GtkWidget *test_widget);
 
 /*
  * tests decalaration
@@ -252,6 +259,24 @@ build_tests (MainConfig *config)
 	test->signal_conn_changed = NULL;
 	list = g_slist_append (list, test);
 
+	/* GnomeDbDataImport test */
+	test = g_new0 (ATest, 1);
+	test->test_name = "Data import test";
+	test->test_descr = "";
+	test->make_test_widget = import_test_make;
+	test->clean_data = NULL;
+	test->signal_conn_changed = NULL;
+	list = g_slist_append (list, test);
+
+	/* GnomeDbServerOperation test */
+	test = g_new0 (ATest, 1);
+	test->test_name = "Server operation";
+	test->test_descr = "";
+	test->make_test_widget = server_op_test_make;
+	test->clean_data = server_op_test_clean;
+	test->signal_conn_changed = server_op_test_signal;
+	list = g_slist_append (list, test);
+
 	config->tests = list;
 }
 
@@ -306,10 +331,11 @@ main (int argc, char **argv)
 
 	/* Test Configuration */
 	config = g_new0 (MainConfig, 1);
-	dict = GDA_DICT (gda_dict_new ());
+	dict = gda_dict_new ();
+
 	config->dict = dict;
 	build_tests (config);
-	g_object_set (dict, "with_functions", TRUE, NULL);
+	gda_dict_extend_with_functions (dict);
 
 	if (list_tests) {
 		GSList *list = config->tests;
@@ -357,7 +383,14 @@ main (int argc, char **argv)
 		load_file (config);
 	}
 	else {
-		set_filename (config, "TEST_DICT_sqlite.xml");
+		gchar *path, *file;
+
+		path = g_get_current_dir ();
+		file = g_build_filename (path, "TEST_DICT_sqlite.xml", NULL);
+		g_free (path);
+		set_filename (config, file);
+		g_free (file);
+
 		load_file (config);
 		use_local_db = TRUE;
 		conn_open (config);
@@ -642,7 +675,7 @@ sync_action_cb (GtkWidget *wid, MainConfig *config)
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(dlg)->vbox), updview, TRUE, TRUE, 0);
 	gtk_widget_show (dlg);
 	/* data types, functions and aggregates */
-	if (!gda_dict_update_dbms_data (config->dict, &error)) {
+	if (!gda_dict_update_dbms_meta_data (config->dict, 0, NULL, &error)) {
 		if (error->code != GDA_DICT_META_DATA_UPDATE_USER_STOPPED) {
 			GtkWidget *msg;
 			
@@ -657,7 +690,7 @@ sync_action_cb (GtkWidget *wid, MainConfig *config)
 		error = NULL;	
 	}
 	/* database */
-	if (!gda_dict_database_update_dbms_data (db, &error)) {
+	if (!gda_dict_database_update_dbms_data (db, 0, NULL, &error)) {
 		if (error->code != GDA_DICT_DATABASE_META_DATA_UPDATE_USER_STOPPED) {
 			GtkWidget *msg;
 			
@@ -680,7 +713,7 @@ sync_action_cb (GtkWidget *wid, MainConfig *config)
 static void
 stop_dbms_update_cb (GtkWidget *dlg, gint response, MainConfig *config)
 {
-	gda_dict_stop_update_dbms_data (config->dict);
+	gda_dict_stop_update_dbms_meta_data (config->dict);
 }
 
 static void
@@ -735,7 +768,8 @@ conn_open (MainConfig *config)
 		if (use_local_db) {
 			/* use a local SQLite database */
 			gchar *dir, *str;
-			dir = get_current_dir_name ();
+			if ((dir = getcwd(NULL, 0)) == NULL)
+				exit(1);
 			str = g_build_filename (dir, "TEST_sqlite.db", NULL);
 			free (dir);
 			dir = g_strdup_printf ("URI=%s", str);
@@ -743,6 +777,7 @@ conn_open (MainConfig *config)
 			str = dir;
 
 			cnc = gda_client_open_connection_from_string (config->client, "SQLite",
+								      NULL, NULL,
 								      str, 0, &error);
 			g_free (str);
 			use_local_db = FALSE;
@@ -883,7 +918,7 @@ build_page (MainConfig *config)
 /*
  * utility function to provide a new GnomeDbResultSet
  *
- * Returns: NULL if an error occured, or if the user did not enter any required parameter
+ * Returns: NULL if an error occurred, or if the user did not enter any required parameter
  */
 static GdaDataModel *
 get_data_model_from_query (GdaQuery *query, const gchar *ident)
@@ -894,7 +929,7 @@ get_data_model_from_query (GdaQuery *query, const gchar *ident)
 		GdaParameterList *dset;
 		
 		model = gda_data_model_query_new (query);
-		dset = gda_data_model_query_get_param_list (GDA_DATA_MODEL_QUERY (model));
+		dset = gda_data_model_query_get_parameter_list (GDA_DATA_MODEL_QUERY (model));
 		if (dset && !gda_parameter_list_is_valid (dset)) {
 			GtkWidget *dlg;
 			gint status;
@@ -982,14 +1017,20 @@ right_del_clicked_cb (GtkWidget *button, MainConfig *config)
 static void
 param_list_param_changed_cb (GdaParameterList *paramlist, GdaParameter *param, gchar *prefix)
 {
-	const GdaValue *value;
+	const GValue *value;
 	gchar *str;
 
 	value = gda_parameter_get_value (param);
 	if (value)
-		str = gda_value_stringify ((GdaValue *) value);
+		str = gda_value_stringify ((GValue *) value);
 	else
 		str = "_null_";
+	if (strlen (str) > 30) {
+		str [29] = '.';
+		str [28] = '.';
+		str [27] = '.';
+		str [30] = 0;		
+	}
 	g_print ("%s: parameter %p '%s' changed to %s%s\n", prefix,
 		 param, gda_object_get_name (GDA_OBJECT (param)), str,
 		 gda_parameter_is_valid (param) ? "" : " (invalid)");
@@ -1119,11 +1160,11 @@ query_graph_test_graph_changed (GnomeDbSelector *mgsel, GObject *obj, TestData2
 		GdaGraph *graph;
 
 		dict = gda_object_get_dict (GDA_OBJECT (obj));
-		graph = gda_dict_get_graph_for_object (dict, obj);
+		graph = gda_graphs_get_graph_for_object (dict, obj);
 		if (!graph) {
 			/* the query did not have any graph: create one */
 			graph = GDA_GRAPH (gda_graph_query_new (GDA_QUERY (obj)));
-			gda_dict_assume_graph (dict, graph);
+			gda_dict_assume_object (dict, (GdaObject *) graph);
 			g_object_unref (graph);
 		}
 		if (data->canvas)
@@ -1237,7 +1278,7 @@ form_grid_selected_obj_changed (GnomeDbSelector *mgsel, GObject *obj, TestData3
 		GdaDict *dict = gda_object_get_dict (GDA_OBJECT (obj));
 
 		str = g_strdup_printf ("SELECT * FROM %s", gda_object_get_name (GDA_OBJECT (obj)));
-		query = GDA_QUERY (gda_query_new_from_sql (dict, str, NULL));
+		query = gda_query_new_from_sql (dict, str, NULL);
 		g_free (str);
 		g_assert (gda_query_get_query_type (query) == GDA_QUERY_TYPE_SELECT);
 
@@ -1345,6 +1386,11 @@ form_grid_test_signal (MainConfig *config, GtkWidget *test_widget)
 typedef struct {
 	GtkWidget  *vbox;
 	GtkWidget  *wid;
+
+	gboolean          sync;
+	GdaDataModelIter *grid_iter;
+	GdaDataModelIter *form_iter;
+	GtkWidget        *grid_treeview;
 } TestData4;
 
 static GtkWidget *cust_forms_grids_test_prepare (MainConfig *config, TestData4 *data);
@@ -1367,6 +1413,41 @@ cust_forms_grids_test_make (MainConfig *config)
 	return ret;	
 }
 
+static void
+cust_forms_grids_test_sync_selections_cb (GtkToggleButton *toggle, TestData4 *data)
+{
+	data->sync = gtk_toggle_button_get_active (toggle);
+}
+
+static void
+cust_forms_grids_test_sel_row_changed_cb (GdaDataModelIter *iter, gint row, TestData4 *data)
+{
+	GdaDataModelIter *other;
+
+	if (iter == data->grid_iter)
+		other = data->form_iter;
+	else
+		other = data->grid_iter;
+
+	if (data->sync) {
+		/* 2 methods to sync data */
+		if (TRUE || (other == data->form_iter)) {
+			g_signal_handlers_block_by_func (other,
+							 G_CALLBACK (cust_forms_grids_test_sel_row_changed_cb), 
+							 data);
+			gda_data_model_iter_set_at_row (other, row);
+			g_signal_handlers_unblock_by_func (other,
+							   G_CALLBACK (cust_forms_grids_test_sel_row_changed_cb), 
+							   data);
+		}
+		else {
+			GtkTreePath *path = gtk_tree_path_new_from_indices (row, -1);
+			gtk_tree_view_set_cursor (data->grid_treeview, path, NULL, FALSE);
+			gtk_tree_path_free (path);
+		}
+	}
+}
+
 static GtkWidget *
 cust_forms_grids_test_prepare (MainConfig *config, TestData4 *data)
 {
@@ -1380,14 +1461,42 @@ cust_forms_grids_test_prepare (MainConfig *config, TestData4 *data)
 	GnomeDbRawForm *raw_form;
 	GdaDataProxy *proxy;
 
-	query = (GdaQuery *) gda_query_new_from_sql (config->dict, 
-						     "SELECT c.id, c.name, c.default_served_by, s.name "
-						     "FROM customers c "
-						     "LEFT JOIN salesrep s ON (s.id=c.default_served_by)", NULL);
-	cust_model = get_data_model_from_query (query, "Customers");
-	g_object_unref (query);
+	query = gda_query_new_from_sql (config->dict, 
+					"SELECT c.id, c.name, c.default_served_by, s.name "
+					"FROM customers c "
+					"LEFT JOIN salesrep s ON (s.id=c.default_served_by)", NULL);
+	cust_model = gda_data_model_query_new (query);
+	if (!gda_data_model_query_set_modification_query (GDA_DATA_MODEL_QUERY (cust_model),
+							  "UPDATE customers set "
+							  "id=##[:name='+0' :type='gint'], "
+							  "name=##[:name='+1' :type='gchararray'], "
+							  "default_served_by=##[:name='+2' :type='gint'] "
+							  "WHERE id=##[:name='-0' :type='gint']", &error)) {
+		g_print ("Error in test code: %s\n",
+			 error && error->message ? error->message : "No detail");
+		g_error_free (error);
+	}
+	if (!gda_data_model_query_set_modification_query (GDA_DATA_MODEL_QUERY (cust_model),
+							  "DELETE from customers "
+							  "WHERE id=##[:name='-0' :type='gint']", &error)) {
+		g_print ("Error in test code: %s\n",
+			 error && error->message ? error->message : "No detail");
+		g_error_free (error);
+	}
+	if (!gda_data_model_query_set_modification_query (GDA_DATA_MODEL_QUERY (cust_model),
+							  "INSERT INTO customers (id, name, default_served_by) "
+							  "values ("
+							  "##[:name='+0' :type='gint'], "
+							  "##[:name='+1' :type='gchararray'], "
+							  "##[:name='+2' :type='gint'])",
+							  &error)) {
+		g_print ("Error in test code: %s\n",
+			 error && error->message ? error->message : "No detail");
+		g_error_free (error);
+	}
+	
 
-	query = (GdaQuery *) gda_query_new_from_sql (config->dict, "SELECT id, name FROM salesrep", NULL);
+	query = gda_query_new_from_sql (config->dict, "SELECT id, name FROM salesrep", NULL);
 	sr_model = get_data_model_from_query (query, "Sales people");
 	g_object_unref (query);
 	
@@ -1403,8 +1512,9 @@ cust_forms_grids_test_prepare (MainConfig *config, TestData4 *data)
 	gtk_widget_show (wid);
 
 	/* grid */
-	wid = gtk_label_new ("<b>1 - GnomeDbGrid</b>");
+	wid = gtk_label_new ("<b>GnomeDbGrid</b>");
 	gtk_label_set_use_markup (GTK_LABEL (wid), TRUE);
+	gtk_misc_set_alignment (GTK_MISC (wid), 0., -1);
 	gtk_box_pack_start (GTK_BOX (vbox), wid, FALSE, TRUE, 0);
 	gtk_widget_show (wid);
 
@@ -1414,6 +1524,8 @@ cust_forms_grids_test_prepare (MainConfig *config, TestData4 *data)
 	g_object_get (G_OBJECT (wid), "raw_grid", &raw_grid, NULL);
 
 	iter = gnome_db_data_widget_get_current_data (GNOME_DB_DATA_WIDGET (raw_grid));
+	data->grid_treeview = raw_grid;
+	data->grid_iter = iter;
 	param = g_slist_nth_data (GDA_PARAMETER_LIST (iter)->parameters, 2);
 	if (!gda_parameter_restrict_values (param, sr_model, 0, &error)) {
 		g_print ("Cant' restrict parameter: %s\n", error->message);
@@ -1421,14 +1533,16 @@ cust_forms_grids_test_prepare (MainConfig *config, TestData4 *data)
 	}
 	g_signal_connect (iter, "param_changed",
 			  G_CALLBACK (param_list_param_changed_cb), "Grids & forms + 1 test");
+	g_signal_connect (iter, "row_changed",
+			  G_CALLBACK (cust_forms_grids_test_sel_row_changed_cb), data);
 	gnome_db_data_widget_column_hide (GNOME_DB_DATA_WIDGET (raw_grid), 3);
 
 	proxy = gnome_db_data_widget_get_proxy (GNOME_DB_DATA_WIDGET (raw_grid));
-	/*gda_data_proxy_assign_model_col (proxy, sr_model, 3, 1);*/
 
 	/* form */
-	wid = gtk_label_new ("<b>2 - GnomeDbForm</b>");
+	wid = gtk_label_new ("<b>GnomeDbForm</b>");
 	gtk_label_set_use_markup (GTK_LABEL (wid), TRUE);
+	gtk_misc_set_alignment (GTK_MISC (wid), 0., -1);
 	gtk_box_pack_start (GTK_BOX (vbox), wid, FALSE, TRUE, 0);
 	gtk_widget_show (wid);
 
@@ -1438,6 +1552,8 @@ cust_forms_grids_test_prepare (MainConfig *config, TestData4 *data)
 	g_object_get (G_OBJECT (wid), "raw_form", &raw_form, NULL);
 
 	iter = gnome_db_data_widget_get_current_data (GNOME_DB_DATA_WIDGET (raw_form));
+	/* gnome_db_basic_form_show_entries_actions (GNOME_DB_BASIC_FORM (raw_form), TRUE); */
+	data->form_iter = iter;
 	param = g_slist_nth_data (GDA_PARAMETER_LIST (iter)->parameters, 2);
 	if (!gda_parameter_restrict_values (param, sr_model, 0, &error)) {
 		g_print ("Cant' restrict parameter: %s\n", error->message);
@@ -1445,11 +1561,26 @@ cust_forms_grids_test_prepare (MainConfig *config, TestData4 *data)
 	}
 	g_signal_connect (iter, "param_changed",
 			  G_CALLBACK (param_list_param_changed_cb), "Grids & forms + 2 test");
+	g_signal_connect (iter, "row_changed",
+			  G_CALLBACK (cust_forms_grids_test_sel_row_changed_cb), data);
 	gnome_db_data_widget_column_hide (GNOME_DB_DATA_WIDGET (raw_form), 3);
 
 	g_object_unref (cust_model);
 	g_object_unref (sr_model);
 
+	/* add an option to sync the two's selection */
+	wid = gtk_label_new ("<b>Selections sync. option</b>");
+	gtk_label_set_use_markup (GTK_LABEL (wid), TRUE);
+	gtk_misc_set_alignment (GTK_MISC (wid), 0., -1);
+	gtk_box_pack_start (GTK_BOX (vbox), wid, FALSE, TRUE, 0);
+	gtk_widget_show (wid);
+
+	wid = gtk_check_button_new_with_label ("Keep selections in sync");
+	gtk_box_pack_start (GTK_BOX (vbox), wid, FALSE, TRUE, 0);
+	gtk_widget_show (wid);
+	g_signal_connect (G_OBJECT (wid), "toggled",
+			  G_CALLBACK (cust_forms_grids_test_sync_selections_cb), data);
+
 	return vbox;
 }
 
@@ -1539,14 +1670,14 @@ cust_form_grid_test_prepare (MainConfig *config, TestData5 *data)
 	GdaDataProxy *proxy;
 	GnomeDbDataWidget *data_widget;
 
-	query = (GdaQuery *) gda_query_new_from_sql (config->dict, 
-						     "SELECT c.id, c.name, c.default_served_by, s.name "
-						     "FROM customers c "
-						     "INNER JOIN salesrep s ON (s.id=c.default_served_by)", NULL);
+	query = gda_query_new_from_sql (config->dict, 
+					"SELECT c.id, c.name, c.default_served_by, s.name "
+					"FROM customers c "
+					"INNER JOIN salesrep s ON (s.id=c.default_served_by)", NULL);
 	cust_model = get_data_model_from_query (query, "Customers");
 	g_object_unref (query);
 
-	query = (GdaQuery *) gda_query_new_from_sql (config->dict, "SELECT id, name FROM salesrep", NULL);
+	query = gda_query_new_from_sql (config->dict, "SELECT id, name FROM salesrep", NULL);
 	sr_model = get_data_model_from_query (query, "Sales people");
 	g_object_unref (query);
 	
@@ -1561,8 +1692,10 @@ cust_form_grid_test_prepare (MainConfig *config, TestData5 *data)
 
 	iter = gnome_db_data_widget_get_current_data (data_widget);
 	proxy = gnome_db_data_widget_get_proxy (data_widget);
+#ifdef PROXY_STORE_EXTRA_VALUES
 	gda_data_proxy_assign_model_col (proxy, sr_model, 3, 1);
-
+#endif
+	
 	param = g_slist_nth_data (GDA_PARAMETER_LIST (iter)->parameters, 2);
 	if (!gda_parameter_restrict_values (param, sr_model, 0, &error)) {
 		g_print ("Cant' restrict parameter: %s\n", error->message);
@@ -1664,7 +1797,7 @@ forms_grids_selected_obj_changed (GnomeDbSelector *mgsel, GObject *obj, TestData
 		GdaDict *dict = gda_object_get_dict (GDA_OBJECT (obj));
 
 		str = g_strdup_printf ("SELECT * FROM %s", gda_object_get_name (GDA_OBJECT (obj)));
-		query = GDA_QUERY (gda_query_new_from_sql (dict, str, NULL));
+		query = gda_query_new_from_sql (dict, str, NULL);
 		g_free (str);
 		g_assert (gda_query_get_query_type (query) == GDA_QUERY_TYPE_SELECT);
 
@@ -2065,6 +2198,18 @@ console_test_make (MainConfig *config)
 	return sw;
 }
 
+/*
+ * GnomeDbDataImport test
+ */
+static GtkWidget *
+import_test_make (MainConfig *config)
+{
+	GtkWidget *import;
+	
+	import = gnome_db_data_import_new (config->dict);
+	return import;
+}
+
 /*
  * GnomeDbCombo test
  */
@@ -2117,7 +2262,7 @@ combo_test_combo_changed (GnomeDbSelector *mgsel, GObject *obj, TestData8 *data)
 		GdaDict *dict = gda_object_get_dict (GDA_OBJECT (obj));
 
 		str = g_strdup_printf ("SELECT * FROM %s", gda_object_get_name (GDA_OBJECT (obj)));
-		query = GDA_QUERY (gda_query_new_from_sql (dict, str, NULL));
+		query = gda_query_new_from_sql (dict, str, NULL);
 		g_free (str);
 		g_assert (gda_query_get_query_type (query) == GDA_QUERY_TYPE_SELECT);
 
@@ -2351,7 +2496,7 @@ xml_spec_button_clicked_cb (GtkWidget *button, MainConfig *config)
 
 	gtk_text_buffer_get_bounds (data->buffer, &start, &end);
 	str = gtk_text_buffer_get_text (data->buffer, &start, &end, FALSE);
-	dset = GDA_PARAMETER_LIST (gda_parameter_list_new_from_spec (config->dict, str, &error));
+	dset = GDA_PARAMETER_LIST (gda_parameter_list_new_from_spec_string (config->dict, str, &error));
 	g_free (str);
 	if (dset) {
 		GtkWidget *form;
@@ -2400,3 +2545,430 @@ xml_spec_test_clean (MainConfig *config, GtkWidget *test_widget)
 	if (data) 
 		g_free (data);
 }
+
+/*
+ *
+ * ServerOperation test
+ *
+ */
+typedef struct {
+	MainConfig              *config;
+	GtkWidget               *form_vbox;
+	GdaServerOperation      *op;
+	GtkWidget               *op_form;
+	GnomeDbProviderSelector *prov_sel;
+	GtkWidget               *filesel;
+	GtkWidget               *op_combo;
+	gulong                   signal_handler;
+	GdaServerProvider       *prov;
+} TestData10;
+
+static void server_op_test_do_test (GtkButton *button, TestData10 *data);
+static void server_op_test_do_dump (GtkButton *button, TestData10 *data);
+static void server_op_test_do_load (GtkButton *button, TestData10 *data);
+static void server_op_test_do_sql (GtkButton *button, TestData10 *data);
+static void server_op_test_do_perform (GtkButton *button, TestData10 *data);
+static void server_op_test_update_model (TestData10 *data);
+static void server_op_test_changed (GnomeDbSelector *mgsel, GObject *obj, TestData10 *data);
+static GdaServerProvider *server_op_test_get_provider_obj (TestData10 *data);
+static void server_op_test_provider_changed_cb (GnomeDbProviderSelector *prov_sel, TestData10 *data);
+
+
+static GtkWidget *
+server_op_test_make (MainConfig *config)
+{
+	GtkWidget *wid;
+	TestData10 *data;
+	GtkWidget *table;
+
+	/* private test data */
+	data = g_new0 (TestData10, 1);
+	data->op_form = NULL;
+	data->config = config;
+
+	data->form_vbox = gtk_vbox_new (FALSE, 0);
+	g_object_set_data (G_OBJECT (data->form_vbox), "private_data", data);
+
+	/* a box for text and button */
+	table = gtk_table_new (3, 5, FALSE);
+	gtk_box_pack_start (GTK_BOX (data->form_vbox), table, FALSE, TRUE, 0);
+	gtk_widget_show (table);
+
+	/* provider selection */
+	wid = gtk_label_new ("Provider:");
+	gtk_table_attach (GTK_TABLE (table), wid, 0, 1, 0, 1, 0, 0, 0, 0);
+	gtk_widget_show (wid);
+
+	wid = gnome_db_provider_selector_new ();
+	gnome_db_provider_selector_set_selected_provider (GNOME_DB_PROVIDER_SELECTOR (wid),
+							  "PostgreSQL");
+	gtk_table_attach_defaults (GTK_TABLE (table), wid, 1, 2, 0, 1);
+	gtk_widget_show (wid);
+	data->prov_sel = GNOME_DB_PROVIDER_SELECTOR (wid);
+	g_signal_connect (G_OBJECT (wid), "changed", 
+			  G_CALLBACK (server_op_test_provider_changed_cb), data);
+
+	/* operation selector */
+	wid = gtk_label_new ("Operation:");
+	gtk_table_attach (GTK_TABLE (table), wid, 2, 3, 0, 1, 0, 0, 0, 0);
+	gtk_widget_show (wid);
+
+	wid = gnome_db_combo_new ();
+	gtk_table_attach_defaults (GTK_TABLE (table), wid, 3, 4, 0, 1);
+	gtk_widget_show (wid);
+	data->op_combo = wid;
+
+	/* test! */
+	wid = gtk_button_new_with_label ("Test!");
+	gtk_table_attach (GTK_TABLE (table), wid, 0, 1, 1, 2, 0, 0, 0, 0);
+	g_signal_connect (G_OBJECT (wid), "clicked",
+			  G_CALLBACK (server_op_test_do_test), data);
+	gtk_widget_show (wid);
+
+	/* dump */
+	wid = gtk_button_new_with_label ("Dump");
+	gtk_table_attach (GTK_TABLE (table), wid, 1, 2, 1, 2, 0, 0, 0, 0);
+	g_signal_connect (G_OBJECT (wid), "clicked",
+			  G_CALLBACK (server_op_test_do_dump), data);
+	gtk_widget_show (wid);
+
+	/* render SQL */
+	wid = gtk_button_new_with_label ("SQL");
+	gtk_table_attach (GTK_TABLE (table), wid, 2, 3, 1, 2, 0, 0, 0, 0);
+	g_signal_connect (G_OBJECT (wid), "clicked",
+			  G_CALLBACK (server_op_test_do_sql), data);
+	gtk_widget_show (wid);
+
+	/* Perform operation */
+	wid = gtk_button_new_with_label ("Perform!");
+	gtk_table_attach (GTK_TABLE (table), wid, 3, 4, 1, 2, 0, 0, 0, 0);
+	g_signal_connect (G_OBJECT (wid), "clicked",
+			  G_CALLBACK (server_op_test_do_perform), data);
+	gtk_widget_show (wid);
+
+	/* load data */
+	wid = gtk_label_new ("Load data:");
+	gtk_table_attach (GTK_TABLE (table), wid, 0, 1, 2, 3, 0, 0, 0, 0);
+	gtk_widget_show (wid);
+
+	wid = gtk_file_chooser_button_new ("File to load into the GdaServerOperation object", 
+					   GTK_FILE_CHOOSER_ACTION_OPEN);
+	data->filesel = wid;
+	gtk_table_attach_defaults (GTK_TABLE (table), wid, 1, 2, 2, 3);
+	gtk_widget_show (wid);
+
+	wid = gtk_button_new_with_label ("Load");
+	gtk_table_attach (GTK_TABLE (table), wid, 2, 3, 2, 3, 0, 0, 0, 0);
+	g_signal_connect (G_OBJECT (wid), "clicked",
+			  G_CALLBACK (server_op_test_do_load), data);
+	gtk_widget_show (wid);
+
+	data->signal_handler = g_signal_connect (G_OBJECT (config->selector), "selection_changed",
+						 G_CALLBACK (server_op_test_changed), data);
+	gtk_widget_show (data->form_vbox);
+
+	return data->form_vbox;
+
+}
+
+static void
+server_op_test_changed (GnomeDbSelector *mgsel, GObject *obj, TestData10 *data)
+{
+	server_op_test_update_model (data);
+}
+
+static void
+server_op_test_provider_changed_cb (GnomeDbProviderSelector *prov_sel, TestData10 *data)
+{
+	if (data->prov) {
+		g_object_unref (data->prov);
+		data->prov = NULL;
+	}
+	server_op_test_update_model (data);
+}
+
+static void
+server_op_test_update_model (TestData10 *data)
+{
+	GdaParameter *param = NULL;
+	GdaParameterList *options = NULL;
+	GdaServerOperationType type;
+	GdaDataModel *model;
+	GObject *obj;
+	
+	obj = gnome_db_selector_get_selected_object (GNOME_DB_SELECTOR (data->config->selector));
+	if (GDA_IS_DICT_TABLE (obj)) {
+		if (gda_dict_table_is_view (GDA_DICT_TABLE (obj)))
+			param = gda_parameter_new_string ("VIEW", gda_object_get_name (GDA_OBJECT (obj)));
+		else
+			param = gda_parameter_new_string ("TABLE", gda_object_get_name (GDA_OBJECT (obj)));
+	}
+		
+	if (param) {
+		options = gda_parameter_list_new (NULL);
+		gda_parameter_list_add_param (options, param);
+		g_object_unref (param);
+	}
+
+	model = gnome_db_combo_get_model (GNOME_DB_COMBO (data->op_combo));
+	if (!model) {
+		gint columns[] = {1};
+		model = gda_data_model_array_new_with_g_types (2, G_TYPE_INT, G_TYPE_STRING);
+		
+		gnome_db_combo_set_model (GNOME_DB_COMBO (data->op_combo), model, 1, columns);
+	}
+	else
+		gda_data_model_array_clear (GDA_DATA_MODEL_ARRAY (model));
+	for (type = 0; type < GDA_SERVER_OPERATION_NB; type ++)
+		if (gda_server_provider_supports_operation (server_op_test_get_provider_obj (data),
+							    gda_dict_get_connection (data->config->dict), 
+							    type, options)) {
+			gint row;
+
+			row = gda_data_model_append_row (model, NULL);
+			if (row < 0)
+				g_error ("Cant' append data to a GdaDataModelArray");
+			else {
+				GValue value;
+
+				memset (&value, 0, sizeof (GValue));
+				g_value_init (&value, G_TYPE_INT);
+				g_value_set_int (&value, type);
+				gda_data_model_set_value_at (model, 0, row, &value, NULL);
+
+				memset (&value, 0, sizeof (GValue));
+				g_value_init (&value, G_TYPE_STRING);
+				g_value_set_string (&value, gda_server_operation_op_type_to_string (type));
+				gda_data_model_set_value_at (model, 1, row, &value, NULL);
+			}
+		}
+	if (options)
+		g_object_unref (options);
+}
+
+static GdaServerProvider *
+server_op_test_get_provider_obj (TestData10 *data)
+{
+	GdaConnection *cnc;
+	GdaServerProvider *prov = NULL;
+
+	cnc = gda_dict_get_connection (data->config->dict);
+	if (cnc && gda_connection_is_opened (cnc))
+		return gda_connection_get_provider_obj (cnc);
+
+	if (data->prov)
+		prov = data->prov;
+	else {
+		/* create the GdaServerProvider object */
+		data->prov = gnome_db_provider_selector_get_selected_provider_obj (data->prov_sel);
+		prov = data->prov;
+	}
+
+	return prov;
+}
+
+static void
+server_op_test_do_test (GtkButton *button, TestData10 *data)
+{
+	GdaServerProvider *prov = NULL;
+	GdaServerOperationType type;
+	GError *error = NULL;
+	GSList *combo_values;
+	gint columns[] = {0};
+
+	if (data->op) {
+		g_object_unref (data->op);
+		data->op = NULL;
+	}
+	if (data->op_form) 
+		gtk_widget_destroy (data->op_form);
+
+	combo_values = gnome_db_combo_get_values_ext (GNOME_DB_COMBO (data->op_combo), 1, columns);
+	if (!combo_values || !gda_value_isa ((GValue *) (combo_values->data), G_TYPE_INT)) {
+		GtkWidget *label;
+
+		label = gtk_label_new ("Select an operation to perform");
+		gtk_box_pack_start (GTK_BOX (data->form_vbox), label, TRUE, TRUE, 10);
+		data->op_form = label;
+		return;
+	}
+	type = g_value_get_int ((GValue *) (combo_values->data));
+	g_slist_free (combo_values);
+	
+	prov = server_op_test_get_provider_obj (data);
+	if (prov)
+		data->op = gda_server_provider_create_operation (prov, gda_dict_get_connection (data->config->dict), type, NULL, &error);
+
+	if (!data->op) {
+		GtkWidget *label;
+		gchar *str;
+
+		str = g_strdup_printf ("Can't create GdaServerOperation widget: %s",
+				       error && error->message ? error->message : "No detail");
+		label = gtk_label_new (str);
+		g_free (str);
+		gtk_box_pack_start (GTK_BOX (data->form_vbox), label, TRUE, TRUE, 10);
+		data->op_form = label;
+	} 
+	else {
+		data->op_form = gnome_db_server_operation_new (data->op);
+		gtk_box_pack_start (GTK_BOX (data->form_vbox), data->op_form, TRUE, TRUE, 0);
+	}
+	gtk_widget_show (data->op_form);
+}
+
+static void
+server_op_test_do_dump (GtkButton *button, TestData10 *data)
+{
+	xmlNodePtr node;
+
+	if (!data->op || !data->op_form || ! IS_GNOME_DB_SERVER_OPERATION (data->op_form)) {
+		g_print ("No form!\n");
+		return;
+	}
+
+	node = gda_server_operation_save_data_to_xml (data->op, NULL);
+	if (node) {
+		xmlDocPtr doc;
+		xmlChar *buffer;
+		
+		doc = xmlNewDoc ("1.0");
+		xmlDocSetRootElement (doc, node);
+		xmlIndentTreeOutput = 1;
+		xmlKeepBlanksDefault (0);
+		xmlDocDumpFormatMemory (doc, &buffer, NULL, 1);
+		g_print ("%s\n", buffer);
+		xmlFree (buffer);
+		xmlFreeDoc (doc);
+	}
+	else 
+		g_warning ("Saving to XML failed!");
+}
+
+static void
+server_op_test_do_load (GtkButton *button, TestData10 *data)
+{
+	xmlNodePtr node;
+	xmlDocPtr doc;
+	GError *error = NULL;
+	gchar *filename;
+
+	if (!data->op || !data->op_form || ! IS_GNOME_DB_SERVER_OPERATION (data->op_form)) {
+		g_print ("No form!\n");
+		return;
+	}
+	
+	filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (data->filesel));
+	if (!filename) {
+		g_print ("No file selected to load\n");
+		return;
+	}
+
+	doc = xmlParseFile (filename);
+	if (!doc) {
+		g_print ("Can't load XML file '%s'\n", filename);
+		g_free (filename);
+		return;
+	}
+	g_free (filename);
+
+	node = xmlDocGetRootElement (doc);
+	if (!gda_server_operation_load_data_from_xml (data->op, node, &error)) {
+		g_print ("Can't load data: %s\n", error && error->message ? error->message : "No detail");
+		if (error)
+			g_error_free (error);
+	}
+
+	xmlFreeDoc (doc);
+}
+
+static void
+server_op_test_do_sql (GtkButton *button, TestData10 *data)
+{
+	GdaServerProvider *prov;
+
+	if (!data->op) {
+		g_print ("No GdaServerOperation!\n");
+		return;
+	}
+	prov = server_op_test_get_provider_obj (data);
+	if (prov) {
+		gchar *sql;
+		GError *error = NULL;
+
+		sql = gda_server_provider_render_operation (prov, gda_dict_get_connection (data->config->dict), data->op, &error);
+		if (!sql) {
+			g_print ("Can't render operation: %s\n", error && error->message ? error->message : "No detail");
+			if (error)
+				g_error_free (error);
+		}
+		else {
+			g_print ("SQL: %s\n", sql);
+			g_free (sql);
+		}
+	}
+	else 
+		g_warning ("Could not get provider object");
+}
+
+static void
+server_op_test_do_perform (GtkButton *button, TestData10 *data)
+{
+	GdaServerProvider *prov;
+
+	if (!data->op) {
+		g_print ("No GdaServerOperation!\n");
+		return;
+	}
+	prov = server_op_test_get_provider_obj (data);
+	if (prov) {
+		GError *error = NULL;
+
+		if (!gda_server_provider_perform_operation (prov, 
+							    gda_dict_get_connection (data->config->dict), 
+							    data->op, &error)) {
+			g_print ("Can't perform operation: %s\n", error && error->message ? error->message : "No detail");
+			if (error)
+				g_error_free (error);
+		}
+		else 
+			g_print ("Operation done!\n");
+	}
+	else 
+		g_warning ("Could not get provider object");
+}
+
+static void
+server_op_test_clean (MainConfig *config, GtkWidget *test_widget)
+{
+	TestData10 *data = g_object_get_data (G_OBJECT (test_widget), "private_data");
+
+	if (data) {
+		g_signal_handler_disconnect (G_OBJECT (config->selector), data->signal_handler);
+		
+		if (data->op)
+			g_object_unref (data->op);
+		g_free (data);
+	}
+}
+
+static gboolean
+server_op_test_signal (MainConfig *config, GtkWidget *test_widget)
+{
+	TestData10 *data = g_object_get_data (G_OBJECT (test_widget), "private_data");
+	GdaConnection *cnc;
+
+	cnc = gda_dict_get_connection (config->dict);
+	gtk_widget_set_sensitive (GTK_WIDGET (data->prov_sel), !cnc || !gda_connection_is_opened (cnc));
+	if (cnc && gda_connection_is_opened (cnc)) {
+		GdaServerProvider *prov;
+		GdaServerProviderInfo* pinfo;
+
+		prov = gda_connection_get_provider_obj (cnc);
+		pinfo = gda_server_provider_get_info (prov, cnc);
+		gnome_db_provider_selector_set_selected_provider (GNOME_DB_PROVIDER_SELECTOR (data->prov_sel),
+								  pinfo->provider_name);
+	}
+
+	return TRUE;
+}
